# ====================================================================
# Copyright (c) 2004-2005 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions: 
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software. 
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#

import re

from unittest import TestCase, main
from cStringIO import StringIO
from PyLucene import *


class SortTestCase(TestCase):
    """
    Unit tests for sorting code, ported from Java Lucene
    """

    # document data:
    # the tracer field is used to determine which document was hit
    # the contents field is used to search and sort by relevance
    # the int field to sort by int
    # the float field to sort by float
    # the string field to sort by string

    data = [
    #     tracer  contents          int           float          string  custom
        [   "A",   "x a",           "5",           "4f",           "c",   "A-3"   ],
        [   "B",   "y a",           "5",           "3.4028235E38", "i",   "B-10"  ],
        [   "C",   "x a b c",       "2147483647",  "1.0",          "j",   "A-2"   ],
        [   "D",   "y a b c",       "-1",          "0.0f",         "a",   "C-0"   ],
        [   "E",   "x a b c d",     "5",           "2f",           "h",   "B-8"   ],
        [   "F",   "y a b c d",     "2",           "3.14159f",     "g",   "B-1"   ],
        [   "G",   "x a b c d",     "3",           "-1.0",         "f",   "C-100" ],
        [   "H",   "y a b c d",     "0",           "1.4E-45",      "e",   "C-88"  ],
        [   "I",   "x a b c d e f", "-2147483648", "1.0e+0",       "d",   "A-10"  ],
        [   "J",   "y a b c d e f", "4",           ".5",           "b",   "C-7"   ],
	[   "W",   "g",             "1",           None,           None,  None    ],
	[   "X",   "g",             "1",           "0.1",          None,  None    ],
	[   "Y",   "g",             "1",           "0.2",          None,  None    ],
	[   "Z",   "f g",           None,          None,           None,  None    ]
        ]


    def _getIndex(self, even, odd):
        """
        Create an index of all the documents, or just the x,
        or just the y documents
        """
        
        indexStore = RAMDirectory()
        writer = IndexWriter(indexStore, SimpleAnalyzer(), True)

        for i in xrange(0, len(self.data)):
            if i % 2 == 0 and even or i % 2 == 1 and odd:
                doc = Document()
                doc.add(Field("tracer", self.data[i][0],
                              Field.Store.YES, Field.Index.NO))
                doc.add(Field("contents", self.data[i][1],
                              Field.Store.NO, Field.Index.TOKENIZED))
                if self.data[i][2] is not None:
                    doc.add(Field("int", self.data[i][2],
                                  Field.Store.NO, Field.Index.UN_TOKENIZED))
                if self.data[i][3] is not None:
                    doc.add(Field("float", self.data[i][3],
                                  Field.Store.NO, Field.Index.UN_TOKENIZED))
                if self.data[i][4] is not None:
                    doc.add(Field("string", self.data[i][4],
                                  Field.Store.NO, Field.Index.UN_TOKENIZED))
                if self.data[i][5] is not None:
                    doc.add(Field("custom", self.data[i][5],
                                  Field.Store.NO, Field.Index.UN_TOKENIZED))
                writer.addDocument(doc)

        writer.optimize()
        writer.close()

        return IndexSearcher(indexStore)

    def _getFullIndex(self):
        return self._getIndex(True, True)

    def _getXIndex(self):
        return self._getIndex(True, False)

    def _getYIndex(self):
        return self._getIndex(False, True)

    def _getEmptyIndex(self):
        return self._getIndex(False, False)

    def _assertMatches(self, searcher, query, sort, expectedResult):
        """
        Make sure the documents returned by the search match the expected list
        """
        
        result = searcher.search(query, sort)
        buff = StringIO()

        for hit in result:
            buff.write(''.join(hit.getDocument().getValues("tracer")))

        self.assertEqual(expectedResult, buff.getvalue())
        buff.close()

    def _assertMatchesPattern(self, searcher, query, sort, pattern):
        """
        make sure the documents returned by the search match the expected
        list pattern
        """

        result = searcher.search(query, sort)
        buff = StringIO()
        
        for i, doc in result:
            buff.write(''.join(doc.getValues("tracer")))

        self.assert_(re.compile(pattern).match(buff.getvalue()))

    def _getComparatorSource(self):
        return self._getComparator()

    def _getComparable(self, termtext):

        class comparable(object):
            def __init__(self, termText):
                self.string_part, self.int_part = termText.split('-')
                self.int_part = int(self.int_part)
            def compareTo(self, o):
                return (self.string_part.__cmp__(o.string_part) or
                        self.int_part.__cmp__(o.int_part))

        return comparable(termtext)

    def _getComparator(self):

        class comparator(object):

            def getComparable(_self, termText):
                return self._getComparable(termText)

            def newComparator(_self, reader, fieldname):
                 enumerator = reader.terms(Term(fieldname, ""))

                 class comparator(object):
                     def __init__(_self, cache):
                         _self.cache = cache
                     def compare(_self, i, j):
                         return _self.cache[i.doc].compareTo(_self.cache[j.doc])
                     def sortValue(_self, i):
                         return _self.cache[i.doc]
                     def sortType(_self):
                         return SortField.CUSTOM

                 try:
                     cache = self._fillCache(reader, enumerator, fieldname)
                     return comparator(cache)
                 finally:
                     enumerator.close()

        return comparator()

    def _fillCache(self, reader, enumerator, fieldName):
        """
        Returns an array of objects which represent that natural order
        of the term values in the given field.

        @param reader     Terms are in this index.
        @param enumerator Use this to get the term values and TermDocs.
        @param fieldname  Comparables should be for this field.
        @return Array of objects representing natural order of terms in field.
        """

        retArray = [None] * reader.maxDoc()
        if len(retArray) > 0:
            termDocs = reader.termDocs()
            try:
                if enumerator.term() is None:
                    raise RuntimeError, "no terms in field " + fieldName
                while True:
                    term = enumerator.term()
                    if term.field() != fieldName:
                        break
                    termval = self._getComparable(term.text())
                    termDocs.seek(enumerator)
                    while termDocs.next():
                        retArray[termDocs.doc()] = termval
                    if not enumerator.next():
                        break
            finally:
                termDocs.close()

        return retArray

    def _runMultiSorts(self, multi):
        """
        runs a variety of sorts useful for multisearchers
        """

        sort = Sort()

        sort.setSort(SortField.FIELD_DOC)
        self._assertMatchesPattern(multi, self.queryA, sort,
                                   "[AB]{2}[CD]{2}[EF]{2}[GH]{2}[IJ]{2}")

        sort.setSort(SortField("int", SortField.INT))
        self._assertMatchesPattern(multi, self.queryA, sort,
                                   "IDHFGJ[ABE]{3}C")
        
        sort.setSort([SortField("int", SortField.INT),
                      SortField.FIELD_DOC])
        self._assertMatchesPattern(multi, self.queryA, sort,
                                   "IDHFGJ[AB]{2}EC")

        sort.setSort("int")
        self._assertMatchesPattern(multi, self.queryA, sort,
                                   "IDHFGJ[AB]{2}EC")

        sort.setSort([SortField("float", SortField.FLOAT),
                      SortField.FIELD_DOC])
        self._assertMatchesPattern(multi, self.queryA, sort,
                                   "GDHJ[CI]{2}EFAB")

        sort.setSort("float")
        self._assertMatchesPattern(multi, self.queryA, sort,
                                   "GDHJ[CI]{2}EFAB")

        sort.setSort("string")
        self._assertMatches(multi, self.queryA, sort, "DJAIHGFEBC")

        sort.setSort("int", True)
        self._assertMatchesPattern(multi, self.queryA, sort,
                                   "C[AB]{2}EJGFHDI")

        sort.setSort("float", True)
        self._assertMatchesPattern(multi, self.queryA, sort,
                                   "BAFE[IC]{2}JHDG")

        sort.setSort("string", True)
        self._assertMatches(multi, self.queryA, sort, "CBEFGHIAJD")

        sort.setSort([SortField("string", Locale.US)])
        self._assertMatches(multi, self.queryA, sort, "DJAIHGFEBC")

        sort.setSort([SortField("string", Locale.US, True)])
        self._assertMatches(multi, self.queryA, sort, "CBEFGHIAJD")

        sort.setSort(["int", "float"])
        self._assertMatches(multi, self.queryA, sort, "IDHFGJEABC")

        sort.setSort(["float", "string"])
        self._assertMatches(multi, self.queryA, sort, "GDHJICEFAB")

        sort.setSort("int")
        self._assertMatches(multi, self.queryF, sort, "IZJ")

        sort.setSort("int", True)
        self._assertMatches(multi, self.queryF, sort, "JZI")

        sort.setSort("float")
        self._assertMatches(multi, self.queryF, sort, "ZJI")

        sort.setSort("string")
        self._assertMatches(multi, self.queryF, sort, "ZJI")

        sort.setSort("string", True)
        self._assertMatches(multi, self.queryF, sort, "IJZ")

    def _getScores(self, hits):

        scoreMap = {}

        for i, doc in hits:
            v = doc.getValues("tracer")
            self.assertEqual(len(v), 1)
            scoreMap[v[0]] = float(hits.score(i))

        return scoreMap

    def _assertSameValues(self, m1, m2):
        """
        make sure all the values in the maps match
        """

        n = len(m1)
        m = len(m2)
        self.assertEqual(n, m)

        for key in m1.iterkeys():
            self.assertEqual(m1[key], m2[key])

    def setUp(self):

        self.full = self._getFullIndex()
        self.searchX = self._getXIndex()
        self.searchY = self._getYIndex()
        self.queryX = TermQuery(Term("contents", "x"))
        self.queryY = TermQuery(Term("contents", "y"))
        self.queryA = TermQuery(Term("contents", "a"))
        self.queryF = TermQuery(Term("contents", "f"))

    def testBuiltInSorts(self):
        """
        test the sorts by score and document number
        """

        sort = Sort()
        self._assertMatches(self.full, self.queryX, sort, "ACEGI")
        self._assertMatches(self.full, self.queryY, sort, "BDFHJ")

        sort.setSort(SortField.FIELD_DOC)
        self._assertMatches(self.full, self.queryX, sort, "ACEGI")
        self._assertMatches(self.full, self.queryY, sort, "BDFHJ")

    def testTypedSort(self):
        """
        test sorts where the type of field is specified
        """

        sort = Sort()
        sort.setSort([SortField("int", SortField.INT),
                      SortField.FIELD_DOC])
        self._assertMatches(self.full, self.queryX, sort, "IGAEC")
        self._assertMatches(self.full, self.queryY, sort, "DHFJB")

        sort.setSort([SortField("float", SortField.FLOAT),
                      SortField.FIELD_DOC])
        self._assertMatches(self.full, self.queryX, sort, "GCIEA")
        self._assertMatches(self.full, self.queryY, sort, "DHJFB")

        sort.setSort([SortField("string", SortField.STRING),
                      SortField.FIELD_DOC])
        self._assertMatches(self.full, self.queryX, sort, "AIGEC")
        self._assertMatches(self.full, self.queryY, sort, "DJHFB")


    def testEmptyIndex(self):
        """
        test sorts when there's nothing in the index
        """

        sort = Sort()
        empty = self._getEmptyIndex()

        self._assertMatches(empty, self.queryX, sort, "")

        sort.setSort(SortField.FIELD_DOC)
        self._assertMatches(empty, self.queryX, sort, "")

        sort.setSort([SortField("int", SortField.INT),
                      SortField.FIELD_DOC])
        self._assertMatches(empty, self.queryX, sort, "")

        sort.setSort([SortField("string", SortField.STRING, True),
                      SortField.FIELD_DOC])
        self._assertMatches(empty, self.queryX, sort, "")

        sort.setSort([SortField("float", SortField.FLOAT),
                      SortField("string", SortField.STRING)])
        self._assertMatches(empty, self.queryX, sort, "")


    def testAutoSort(self):
        """
        test sorts where the type of field is determined dynamically
        """

        sort = Sort()

        sort.setSort("int")
        self._assertMatches(self.full, self.queryX, sort, "IGAEC")
        self._assertMatches(self.full, self.queryY, sort, "DHFJB")

        sort.setSort("float")
        self._assertMatches(self.full, self.queryX, sort, "GCIEA")
        self._assertMatches(self.full, self.queryY, sort, "DHJFB")

        sort.setSort("string")
        self._assertMatches(self.full, self.queryX, sort, "AIGEC")
        self._assertMatches(self.full, self.queryY, sort, "DJHFB")

    def testReverseSort(self):
        """
        test sorts in reverse
        """

        sort = Sort()

        sort.setSort([SortField(None, SortField.SCORE, True),
                      SortField.FIELD_DOC])
        self._assertMatches(self.full, self.queryX, sort, "IEGCA")
        self._assertMatches(self.full, self.queryY, sort, "JFHDB")

        sort.setSort(SortField(None, SortField.DOC, True))
        self._assertMatches(self.full, self.queryX, sort, "IGECA")
        self._assertMatches(self.full, self.queryY, sort, "JHFDB")

        sort.setSort("int", True)
        self._assertMatches(self.full, self.queryX, sort, "CAEGI")
        self._assertMatches(self.full, self.queryY, sort, "BJFHD")

        sort.setSort("float", True)
        self._assertMatches(self.full, self.queryX, sort, "AECIG")
        self._assertMatches(self.full, self.queryY, sort, "BFJHD")

        sort.setSort("string", True)
        self._assertMatches(self.full, self.queryX, sort, "CEGIA")
        self._assertMatches(self.full, self.queryY, sort, "BFHJD")

    def testEmptyFieldSort(self):
        """
        test sorting when the sort field is empty (undefined)
        for some of the documents
        """

        sort = Sort()
        
        sort.setSort("string")
        self._assertMatches(self.full, self.queryF, sort, "ZJI")

        sort.setSort("string", True)
        self._assertMatches(self.full, self.queryF, sort, "IJZ")

        sort.setSort("int")
        self._assertMatches(self.full, self.queryF, sort, "IZJ")

        sort.setSort("int", True)
        self._assertMatches(self.full, self.queryF, sort, "JZI")

        sort.setSort("float")
        self._assertMatches(self.full, self.queryF, sort, "ZJI")

        sort.setSort("float", True)
        self._assertMatches(self.full, self.queryF, sort, "IJZ")
        
    def testSortCombos(self):
        """
        test sorts using a series of fields
        """

        sort = Sort()
        
        sort.setSort(["int", "float"])
        self._assertMatches(self.full, self.queryX, sort, "IGEAC")

        sort.setSort([SortField("int", True),
                      SortField(None, SortField.DOC, True)])
        self._assertMatches(self.full, self.queryX, sort, "CEAGI")

        sort.setSort(["float", "string"])
        self._assertMatches(self.full, self.queryX, sort, "GICEA")

    def testLocaleSort(self):
        """
        test using a Locale for sorting strings
        """

        sort = Sort()
        
        sort.setSort([SortField("string", Locale.US)])
        self._assertMatches(self.full, self.queryX, sort, "AIGEC")
        self._assertMatches(self.full, self.queryY, sort, "DJHFB")

        sort.setSort([SortField("string", Locale.US, True)])
        self._assertMatches(self.full, self.queryX, sort, "CEGIA")
        self._assertMatches(self.full, self.queryY, sort, "BFHJD")

    def testCustomSorts(self):
        """
        test a custom sort function
        """

        sort = Sort()
        
        sort.setSort(SortField("custom", self._getComparatorSource()))
        self._assertMatches(self.full, self.queryX, sort, "CAIEG")

        sort.setSort(SortField("custom", self._getComparatorSource(), True))
        self._assertMatches(self.full, self.queryY, sort, "HJDBF")

        custom = self._getComparator()
        sort.setSort(SortField("custom", custom))
        self._assertMatches(self.full, self.queryX, sort, "CAIEG")

        sort.setSort(SortField("custom", custom, True))
        self._assertMatches(self.full, self.queryY, sort, "HJDBF")

    def testMultiSort(self):
        """
        test a variety of sorts using more than one searcher
        """
         
        searcher = MultiSearcher([self.searchX, self.searchY])
        self._runMultiSorts(searcher)

    def testParallelMultiSort(self):
        """
        test a variety of sorts using a parallel multisearcher
        """

        searcher = ParallelMultiSearcher([self.searchX, self.searchY])
        self._runMultiSorts(searcher)

    def testNormalizedScores(self):
        """
        test that the relevancy scores are the same even if
        hits are sorted
        """

        full = self.full

        # capture relevancy scores
        scoresX = self._getScores(full.search(self.queryX))
        scoresY = self._getScores(full.search(self.queryY))
        scoresA = self._getScores(full.search(self.queryA))

        # we'll test searching locally and multi
        # note: the multi test depends on each separate index containing
        # the same documents as our local index, so the computed normalization
        # will be the same.  so we make a multi searcher over two equal document
        # sets - not realistic, but necessary for testing.

        queryX = self.queryX
        queryY = self.queryY
        queryA = self.queryA

        multi = MultiSearcher([self.searchX, self.searchY])

        gs = self._getScores

        # change sorting and make sure relevancy stays the same

        sort = Sort()

        self._assertSameValues(scoresX, gs(full.search(queryX, sort)))
        self._assertSameValues(scoresX, gs(multi.search(queryX, sort)))
        self._assertSameValues(scoresY, gs(full.search(queryY, sort)))
        self._assertSameValues(scoresY, gs(multi.search(queryY, sort)))
        self._assertSameValues(scoresA, gs(full.search(queryA, sort)))
        self._assertSameValues(scoresA, gs(multi.search(queryA, sort)))

        sort.setSort(SortField.FIELD_DOC)
        self._assertSameValues(scoresX, gs(full.search(queryX,sort)))
        self._assertSameValues(scoresX, gs(multi.search(queryX, sort)))
        self._assertSameValues(scoresY, gs(full.search(queryY, sort)))
        self._assertSameValues(scoresY, gs(multi.search(queryY, sort)))
        self._assertSameValues(scoresA, gs(full.search(queryA, sort)))
        self._assertSameValues(scoresA, gs(multi.search(queryA, sort)))

        sort.setSort("int")
        self._assertSameValues(scoresX, gs(full.search(queryX, sort)))
        self._assertSameValues(scoresX, gs(multi.search(queryX, sort)))
        self._assertSameValues(scoresY, gs(full.search(queryY, sort)))
        self._assertSameValues(scoresY, gs(multi.search(queryY, sort)))
        self._assertSameValues(scoresA, gs(full.search(queryA, sort)))
        self._assertSameValues(scoresA, gs(multi.search(queryA, sort)))

        sort.setSort("float")
        self._assertSameValues(scoresX, gs(full.search(queryX, sort)))
        self._assertSameValues(scoresX, gs(multi.search(queryX, sort)))
        self._assertSameValues(scoresY, gs(full.search(queryY, sort)))
        self._assertSameValues(scoresY, gs(multi.search(queryY, sort)))
        self._assertSameValues(scoresA, gs(full.search(queryA, sort)))
        self._assertSameValues(scoresA, gs(multi.search(queryA, sort)))

        sort.setSort("string")
        self._assertSameValues(scoresX, gs(full.search(queryX, sort)))
        self._assertSameValues(scoresX, gs(multi.search(queryX, sort)))
        self._assertSameValues(scoresY, gs(full.search(queryY, sort)))
        self._assertSameValues(scoresY, gs(multi.search(queryY, sort)))
        self._assertSameValues(scoresA, gs(full.search(queryA, sort)))
        self._assertSameValues(scoresA, gs(multi.search(queryA, sort)))

        sort.setSort(["int", "float"])
        self._assertSameValues(scoresX, gs(full.search(queryX, sort)))
        self._assertSameValues(scoresX, gs(multi.search(queryX, sort)))
        self._assertSameValues(scoresY, gs(full.search(queryY, sort)))
        self._assertSameValues(scoresY, gs(multi.search(queryY, sort)))
        self._assertSameValues(scoresA, gs(full.search(queryA, sort)))
        self._assertSameValues(scoresA, gs(multi.search(queryA, sort)))

        sort.setSort([SortField("int", True),
                      SortField(None, SortField.DOC, True)])
        self._assertSameValues(scoresX, gs(full.search(queryX, sort)))
        self._assertSameValues(scoresX, gs(multi.search(queryX, sort)))
        self._assertSameValues(scoresY, gs(full.search(queryY, sort)))
        self._assertSameValues(scoresY, gs(multi.search(queryY, sort)))
        self._assertSameValues(scoresA, gs(full.search(queryA, sort)))
        self._assertSameValues(scoresA, gs(multi.search(queryA, sort)))

        sort.setSort(["float", "string"])
        self._assertSameValues(scoresX, gs(full.search(queryX, sort)))
        self._assertSameValues(scoresX, gs(multi.search(queryX, sort)))
        self._assertSameValues(scoresY, gs(full.search(queryY, sort)))
        self._assertSameValues(scoresY, gs(multi.search(queryY, sort)))
        self._assertSameValues(scoresA, gs(full.search(queryA, sort)))
        self._assertSameValues(scoresA, gs(multi.search(queryA, sort)))


if __name__ == "__main__":
    import sys
    if '-loop' in sys.argv:
        sys.argv.remove('-loop')
        while True:
            try:
                main()
            except:
                pass
    else:
        main()



# ====================================================================
# Copyright (c) 2004-2005 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions: 
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software. 
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#

from unittest import TestCase, main
from PyLucene import *


class DocBoostTestCase(TestCase):
    """
    Unit tests ported from Java Lucene
    """
  
    def testDocBoost(self):

        store = RAMDirectory()
        writer = IndexWriter(store, SimpleAnalyzer(), True)
    
        f1 = Field("field", "word", Field.Store.YES, Field.Index.TOKENIZED)
        f2 = Field("field", "word", Field.Store.YES, Field.Index.TOKENIZED)
        f2.setBoost(2.0)
    
        d1 = Document()
        d2 = Document()
        d3 = Document()
        d4 = Document()
        d3.setBoost(3.0)
        d4.setBoost(2.0)
    
        d1.add(f1)                                 # boost = 1
        d2.add(f2)                                 # boost = 2
        d3.add(f1)                                 # boost = 3
        d4.add(f2)                                 # boost = 4
    
        writer.addDocument(d1)
        writer.addDocument(d2)
        writer.addDocument(d3)
        writer.addDocument(d4)
        writer.optimize()
        writer.close()

        scores = [0.0] * 4

        class hitCollector(object):
            def collect(self, doc, score):
                scores[doc] = score

        IndexSearcher(store).search(TermQuery(Term("field", "word")),
                                    hitCollector())
    
        lastScore = 0.0
        for score in scores:
            self.assert_(score > lastScore)
            lastScore = score


if __name__ == "__main__":
    import sys
    if '-loop' in sys.argv:
        sys.argv.remove('-loop')
        while True:
            try:
                main()
            except:
                pass
    else:
         main()



# ====================================================================
# Copyright (c) 2004-2005 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions: 
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software. 
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#

#
# Author: Andi Vajda
#

from StringIO import StringIO
from HTMLParser import HTMLParser


class InputStreamReader(object):

    def __init__(self, inputStream, encoding):

        super(InputStreamReader, self).__init__()
        self.inputStream = inputStream
        self.encoding = encoding or 'utf-8'

    def _read(self, length):

        return self.inputStream.read(length)

    def read(self, length=-1):

        text = self._read(length)
        text = unicode(text, self.encoding)

        return text

    def close(self):

        self.inputStream.close()


class StringReader(object):

    def __init__(self, unicodeText):

        super(StringReader, self).__init__()
        self.unicodeText = unicode(unicodeText)

    def read(self, length=-1):

        text = self.unicodeText
        if text is None:
            return u''
        
        if length == -1 or length >= len(text):
            self.unicodeText = None
            return text

        text = text[:length]
        self.unicodeText = self.unicodeText[length:]

        return text

    def close(self):
        pass


class HTMLReader(object):

    def __init__(self, reader):

        self.reader = reader

        class htmlParser(HTMLParser):

            def __init__(self):

                HTMLParser.__init__(self)

                self.buffer = StringIO()
                self.position = 0

            def handle_data(self, data):

                self.buffer.write(data)

            def _read(self, length):

                buffer = self.buffer
                size = buffer.tell() - self.position

                if length > 0 and size > length:
                    buffer.seek(self.position)
                    data = buffer.read(length)
                    self.position += len(data)
                    buffer.seek(0, 2)

                elif size > 0:
                    buffer.seek(self.position)
                    data = buffer.read(size)
                    self.position = 0
                    buffer.seek(0)

                else:
                    data = ''

                return data
                
        self.parser = htmlParser()

    def read(self, length=-1):

        while True:
            data = self.reader.read(length)
            if len(data) > 0:
                self.parser.feed(data)
                data = self.parser._read(length)
                if len(data) == 0:
                    continue
            return data

    def close(self):

        self.reader.close()



# ====================================================================
# Copyright (c) 2004-2005 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions: 
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software. 
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#

from unittest import TestCase, main
from PyLucene import *


class TestBooleanQuery(TestCase):
    """
    Unit tests ported from Java Lucene
    """

    def testEquality(self):

        bq1 = BooleanQuery()
        bq1.add(TermQuery(Term("field", "value1")), BooleanClause.Occur.SHOULD)
        bq1.add(TermQuery(Term("field", "value2")), BooleanClause.Occur.SHOULD)

        nested1 = BooleanQuery()
        nested1.add(TermQuery(Term("field", "nestedvalue1")), BooleanClause.Occur.SHOULD)
        nested1.add(TermQuery(Term("field", "nestedvalue2")), BooleanClause.Occur.SHOULD)
        bq1.add(nested1, BooleanClause.Occur.SHOULD)

        bq2 = BooleanQuery()
        bq2.add(TermQuery(Term("field", "value1")), BooleanClause.Occur.SHOULD)
        bq2.add(TermQuery(Term("field", "value2")), BooleanClause.Occur.SHOULD)

        nested2 = BooleanQuery()
        nested2.add(TermQuery(Term("field", "nestedvalue1")), BooleanClause.Occur.SHOULD)
        nested2.add(TermQuery(Term("field", "nestedvalue2")), BooleanClause.Occur.SHOULD)
        bq2.add(nested2, BooleanClause.Occur.SHOULD)
        
        self.assert_(bq1.equals(bq2))


if __name__ == "__main__":
    import sys
    if '-loop' in sys.argv:
        sys.argv.remove('-loop')
        while True:
            try:
                main()
            except:
                pass
    else:
         main()




# sample needs to be rewritten to use RangeFilter

#import os, sys, unittest
#
#baseDir = os.path.dirname(os.path.abspath(sys.argv[0]))
#sys.path.append(baseDir)
#
#import lia.advsearching.FilterTest
#from PyLucene import System
#
#System.setProperty("index.dir", os.path.join(baseDir, 'index'))
#unittest.main(lia.advsearching.FilterTest)



# ====================================================================
# Copyright (c) 2004-2005 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions: 
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software. 
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#

import os

from PyLucene import \
     FSDirectory, Document, Field, IndexWriter, SimpleAnalyzer, System


class VerboseIndexing(object):

    def main(cls, argv):

        vi = VerboseIndexing()
        vi.index()

    def index(self):

        dirPath = os.path.join(System.getProperty("java.io.tmpdir", "tmp"),
                               "verbose-index")
        dir = FSDirectory.getDirectory(dirPath, True)
        writer = IndexWriter(dir, SimpleAnalyzer(), True)

        writer.setInfoStream(System.out)

        for i in xrange(100):
            doc = Document()
            doc.add(Field("keyword", "goober",
                             Field.Store.YES, Field.Index.UN_TOKENIZED))
            writer.addDocument(doc)

        writer.optimize()
        writer.close()

    main = classmethod(main)


if __name__ == "__main__":
    import sys
    VerboseIndexing.main(sys.argv)




# sample needs to be rewritten to use MultiPhraseQuery

#import os, sys, unittest
#
#sys.path.append(os.path.dirname(os.path.abspath(sys.argv[0])))
#
#import lia.advsearching.PhrasePrefixQueryTest
#unittest.main(lia.advsearching.PhrasePrefixQueryTest)



# ====================================================================
# Copyright (c) 2004-2005 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions: 
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software. 
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#

from unittest import TestCase, main
from Streams import StringReader
from PyLucene import *


class PositionIncrementTestCase(TestCase):
    """
    Unit tests ported from Java Lucene
    """

    def testSetPosition(self):

        class _analyzer(object):
            def tokenStream(self, fieldName, reader):
                class _tokenStream(object):
                    def __init__(self):
                        self.TOKENS = ["1", "2", "3", "4", "5"]
                        self.INCREMENTS = [1, 2, 1, 0, 1]
                        self.i = 0
                    def next(self):
                        if self.i == len(self.TOKENS):
                            return None
                        t = Token(self.TOKENS[self.i], self.i, self.i)
                        t.setPositionIncrement(self.INCREMENTS[self.i])
                        self.i += 1
                        return t
                return _tokenStream()

        analyzer = _analyzer()

        store = RAMDirectory()
        writer = IndexWriter(store, analyzer, True)
        d = Document()
        d.add(Field("field", "bogus",
                    Field.Store.YES, Field.Index.TOKENIZED))
        writer.addDocument(d)
        writer.optimize()
        writer.close()

        searcher = IndexSearcher(store)

        q = PhraseQuery()
        q.add(Term("field", "1"))
        q.add(Term("field", "2"))
        hits = searcher.search(q)
        self.assertEqual(0, len(hits))

        q = PhraseQuery()
        q.add(Term("field", "2"))
        q.add(Term("field", "3"))
        hits = searcher.search(q)
        self.assertEqual(1, len(hits))

        q = PhraseQuery()
        q.add(Term("field", "3"))
        q.add(Term("field", "4"))
        hits = searcher.search(q)
        self.assertEqual(0, len(hits))

        q = PhraseQuery()
        q.add(Term("field", "2"))
        q.add(Term("field", "4"))
        hits = searcher.search(q)
        self.assertEqual(1, len(hits))

        q = PhraseQuery()
        q.add(Term("field", "3"))
        q.add(Term("field", "5"))
        hits = searcher.search(q)
        self.assertEqual(1, len(hits))

        q = PhraseQuery()
        q.add(Term("field", "4"))
        q.add(Term("field", "5"))
        hits = searcher.search(q)
        self.assertEqual(1, len(hits))

        q = PhraseQuery()
        q.add(Term("field", "2"))
        q.add(Term("field", "5"))
        hits = searcher.search(q)
        self.assertEqual(0, len(hits))

    def testIncrementingPositions(self):
        """
        Basic analyzer behavior should be to keep sequential terms in one
        increment from one another.
        """
        
        analyzer = WhitespaceAnalyzer()
        ts = analyzer.tokenStream("field",
                                  StringReader(u"one two three four five"))

        while True:
            token = ts.next()
            if token is None:
                break
            self.assertEqual(1, token.getPositionIncrement(),
                             token.termText())


if __name__ == "__main__":
    import sys
    if '-loop' in sys.argv:
        sys.argv.remove('-loop')
        while True:
            try:
                main()
            except:
                pass
    else:
         main()



# ====================================================================
# Copyright (c) 2004-2005 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions: 
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software. 
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#

from lia.common.LiaTestCase import LiaTestCase

from PyLucene import \
     WhitespaceAnalyzer, StandardAnalyzer, Term, QueryParser, Locale, \
     BooleanQuery, FuzzyQuery, IndexSearcher, RangeQuery, TermQuery, \
     BooleanClause


class QueryParserTest(LiaTestCase):

    def setUp(self):

        super(QueryParserTest, self).setUp()
        self.analyzer = WhitespaceAnalyzer()
        self.searcher = IndexSearcher(self.directory)

    def testToString(self):

        query = BooleanQuery()
        query.add(FuzzyQuery(Term("field", "kountry")),
                  BooleanClause.Occur.MUST)
        query.add(TermQuery(Term("title", "western")),
                  BooleanClause.Occur.SHOULD)

        self.assertEqual("+kountry~0.5 title:western",
                         query.toString("field"), "both kinds")

    def testPrefixQuery(self):

        parser = QueryParser("category", StandardAnalyzer())
        parser.setLowercaseExpandedTerms(False)

        print parser.parse("/Computers/technology*").toString("category")

    def testGrouping(self):

        query = QueryParser("subject", self.analyzer).parse("(agile OR extreme) AND methodology")
        hits = self.searcher.search(query)

        self.assertHitsIncludeTitle(hits, "Extreme Programming Explained")
        self.assertHitsIncludeTitle(hits, "The Pragmatic Programmer")

    def testRangeQuery(self):

        parser = QueryParser("subject", self.analyzer) 
        parser.setUseOldRangeQuery(True)

        query = parser.parse("pubmonth:[200401 TO 200412]")

        self.assert_(query.isRangeQuery())

        hits = self.searcher.search(query)
        self.assertHitsIncludeTitle(hits, "Lucene in Action")

        query = QueryParser("pubmonth",
                            self.analyzer).parse("{200201 TO 200208}")

        hits = self.searcher.search(query)
        self.assertEqual(0, len(hits), "JDwA in 200208")
  
    def testDateRangeQuery(self):

        # locale diff between jre and gcj 1/1/04 -> 01/01/04
        # expression = "modified:[1/1/04 TO 12/31/04]"
        
        expression = "modified:[01/01/04 TO 12/31/04]"
        parser = QueryParser("subject", self.analyzer)
        parser.setLocale(Locale.US)
        query = parser.parse(expression)
        print expression, "parsed to", query

        hits = self.searcher.search(query)
        self.assert_(len(hits) > 0)

    def testSlop(self):

        q = QueryParser("field", self.analyzer).parse('"exact phrase"')
        self.assertEqual("\"exact phrase\"", q.toString("field"),
                         "zero slop")

        qp = QueryParser("field", self.analyzer)
        qp.setPhraseSlop(5)
        q = qp.parse('"sloppy phrase"')
        self.assertEqual("\"sloppy phrase\"~5", q.toString("field"),
                         "sloppy, implicitly")

    def testPhraseQuery(self):

        q = QueryParser("field",
                        StandardAnalyzer()).parse('"This is Some Phrase*"')
        self.assertEqual("\"some phrase\"", q.toString("field"), "analyzed")

        q = QueryParser("field", self.analyzer).parse('"term"')
        self.assert_(q.isTermQuery(), "reduced to TermQuery")

    def testLowercasing(self):

        q = QueryParser("field", self.analyzer).parse("PrefixQuery*")
        self.assertEqual("prefixquery*", q.toString("field"), "lowercased")

        qp = QueryParser("field", self.analyzer)
        qp.setLowercaseExpandedTerms(False)
        q = qp.parse("PrefixQuery*")
        self.assertEqual("PrefixQuery*", q.toString("field"), "not lowercased")

    def testWildcard(self):

        try:
            QueryParser("field", self.analyzer).parse("*xyz")
            self.fail("Leading wildcard character should not be allowed")
        except:
            self.assert_(True)

    def testBoost(self):

         q = QueryParser("field", self.analyzer).parse("term^2")
         self.assertEqual("term^2.0", q.toString("field"))

    def testParseException(self):

        try:
            QueryParser("contents", self.analyzer).parse("^&#")
        except:
            # expression is invalid, as expected
            self.assert_(True)
        else:
            self.fail("ParseException expected, but not thrown")

#  public void testStopWord() throws ParseException {
#    Query q = QueryParser.parse("the AND drag", "field",
#        StopAnalyzer())
#    //  QueryParser fails on the previous line - this is a known 
#    //  issue
#  }



# ====================================================================
# Copyright (c) 2004-2005 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions: 
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software. 
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#

from unittest import TestCase

from PyLucene import \
     WhitespaceAnalyzer, IndexSearcher, RAMDirectory, \
     Document, Field, IndexWriter

from lia.extsearch.queryparser.NumberUtils import NumberUtils
from lia.extsearch.queryparser.CustomQueryParser import CustomQueryParser


class AdvancedQueryParserTest(TestCase):

    def setUp(self):

        self.analyzer = WhitespaceAnalyzer()
        self.directory = RAMDirectory()

        writer = IndexWriter(self.directory, self.analyzer, True)

        for i in xrange(1, 501):
            doc = Document()
            doc.add(Field("id", NumberUtils.pad(i),
                          Field.Store.YES, Field.Index.UN_TOKENIZED))
            writer.addDocument(doc)

        writer.close()

    def testCustomQueryParser(self):

        parser = self.analyzer.queryParser(CustomQueryParser(), "field")

        try:
            parser.parse("a?t")
            self.fail("Wildcard queries should not be allowed")
        except:
            # expected
            self.assert_(True)

        try:
            parser.parse("xunit~")
            self.fail("Fuzzy queries should not be allowed")
        except:
            # expected
            self.assert_(True)

    def testCustomMultiFieldQueryParser(self):

        parser = self.analyzer.multiFieldQueryParser(CustomQueryParser(),
                                                     ["field"])

        try:
            parser.parse("a?t")
            self.fail("Wildcard queries should not be allowed")
        except:
            # expected
            self.assert_(True)

        try:
            parser.parse("xunit~")
            self.fail("Fuzzy queries should not be allowed")
        except:
            # expected
            self.assert_(True)

    def testIdRangeQuery(self):

        parser = self.analyzer.queryParser(CustomQueryParser(), "field")

        query = parser.parse("id:[37 TO 346]")
        self.assertEqual("id:[0000000037 TO 0000000346]",
                         query.toString("field"), "padded")

        searcher = IndexSearcher(self.directory)
        hits = searcher.search(query)
        self.assertEqual(310, len(hits))

        print parser.parse("special:[term TO *]")
        print parser.parse("special:[* TO term]")

    def testPhraseQuery(self):

        parser = self.analyzer.queryParser(CustomQueryParser(), "field")

        query = parser.parse("singleTerm")
        self.assert_(query.isTermQuery(), "TermQuery")

        query = parser.parse("\"a phrase\"")
        self.assert_(query.isSpanNearQuery(), "SpanNearQuery")



# ====================================================================
# Copyright (c) 2004-2005 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions: 
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software. 
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#

from unittest import TestCase, main
from PyLucene import *


class PhraseQueryTestCase(TestCase):
    """
    Unit tests ported from Java Lucene
    """

    def setUp(self):

        self.directory = RAMDirectory()
        writer = IndexWriter(self.directory, WhitespaceAnalyzer(), True)
    
        doc = Document()
        doc.add(Field("field", "one two three four five",
                      Field.Store.YES, Field.Index.TOKENIZED))
        writer.addDocument(doc)
    
        writer.optimize()
        writer.close()

        self.searcher = IndexSearcher(self.directory)
        self.query = PhraseQuery()

    def tearDown(self):

        self.searcher.close()
        self.directory.close()

    def testNotCloseEnough(self):

        self.query.setSlop(2)
        self.query.add(Term("field", "one"))
        self.query.add(Term("field", "five"))
        hits = self.searcher.search(self.query)
        self.assertEqual(0, len(hits))

    def testBarelyCloseEnough(self):

        self.query.setSlop(3)
        self.query.add(Term("field", "one"))
        self.query.add(Term("field", "five"))
        hits = self.searcher.search(self.query)
        self.assertEqual(1, len(hits))

    def testExact(self):
        """
        Ensures slop of 0 works for exact matches, but not reversed
        """

        # slop is zero by default
        self.query.add(Term("field", "four"))
        self.query.add(Term("field", "five"))
        hits = self.searcher.search(self.query)
        self.assertEqual(1, len(hits), "exact match")

        self.query = PhraseQuery()
        self.query.add(Term("field", "two"))
        self.query.add(Term("field", "one"))
        hits = self.searcher.search(self.query)
        self.assertEqual(0, len(hits), "reverse not exact")

    def testSlop1(self):

        # Ensures slop of 1 works with terms in order.
        self.query.setSlop(1)
        self.query.add(Term("field", "one"))
        self.query.add(Term("field", "two"))
        hits = self.searcher.search(self.query)
        self.assertEqual(1, len(hits), "in order")

        # Ensures slop of 1 does not work for phrases out of order
        # must be at least 2.
        self.query = PhraseQuery()
        self.query.setSlop(1)
        self.query.add(Term("field", "two"))
        self.query.add(Term("field", "one"))
        hits = self.searcher.search(self.query)
        self.assertEqual(0, len(hits), "reversed, slop not 2 or more")

    def testOrderDoesntMatter(self):
        """
        As long as slop is at least 2, terms can be reversed
        """

        self.query.setSlop(2) # must be at least two for reverse order match
        self.query.add(Term("field", "two"))
        self.query.add(Term("field", "one"))
        hits = self.searcher.search(self.query)
        self.assertEqual(1, len(hits), "just sloppy enough")

        self.query = PhraseQuery()
        self.query.setSlop(2)
        self.query.add(Term("field", "three"))
        self.query.add(Term("field", "one"))
        hits = self.searcher.search(self.query)
        self.assertEqual(0, len(hits), "not sloppy enough")

    def testMulipleTerms(self):
        """
        slop is the total number of positional moves allowed
        to line up a phrase
        """
        
        self.query.setSlop(2)
        self.query.add(Term("field", "one"))
        self.query.add(Term("field", "three"))
        self.query.add(Term("field", "five"))
        hits = self.searcher.search(self.query)
        self.assertEqual(1, len(hits), "two total moves")

        self.query = PhraseQuery()
        self.query.setSlop(5) # it takes six moves to match this phrase
        self.query.add(Term("field", "five"))
        self.query.add(Term("field", "three"))
        self.query.add(Term("field", "one"))
        hits = self.searcher.search(self.query)
        self.assertEqual(0, len(hits), "slop of 5 not close enough")

        self.query.setSlop(6)
        hits = self.searcher.search(self.query)
        self.assertEqual(1, len(hits), "slop of 6 just right")

    def testPhraseQueryWithStopAnalyzer(self):

        directory = RAMDirectory()
        stopAnalyzer = StopAnalyzer()
        writer = IndexWriter(directory, stopAnalyzer, True)
        doc = Document()
        doc.add(Field("field", "the stop words are here",
                      Field.Store.YES, Field.Index.TOKENIZED))
        writer.addDocument(doc)
        writer.close()

        searcher = IndexSearcher(directory)

        # valid exact phrase query
        query = PhraseQuery()
        query.add(Term("field","stop"))
        query.add(Term("field","words"))
        hits = searcher.search(query)
        self.assertEqual(1, len(hits))

        # currently StopAnalyzer does not leave "holes", so this matches.
        query = PhraseQuery()
        query.add(Term("field", "words"))
        query.add(Term("field", "here"))
        hits = searcher.search(query)
        self.assertEqual(1, len(hits))

        searcher.close()
  
    def testPhraseQueryInConjunctionScorer(self):

        directory = RAMDirectory()
        writer = IndexWriter(directory, WhitespaceAnalyzer(), True)
    
        doc = Document()
        doc.add(Field("source", "marketing info",
                      Field.Store.YES, Field.Index.TOKENIZED,
                      Field.TermVector.YES))
        writer.addDocument(doc)
    
        doc = Document()
        doc.add(Field("contents", "foobar",
                      Field.Store.YES, Field.Index.TOKENIZED,
                      Field.TermVector.YES))
        doc.add(Field("source", "marketing info",
                      Field.Store.YES, Field.Index.TOKENIZED,
                      Field.TermVector.YES))
        writer.addDocument(doc)
    
        writer.optimize()
        writer.close()
    
        searcher = IndexSearcher(directory)
    
        phraseQuery = PhraseQuery()
        phraseQuery.add(Term("source", "marketing"))
        phraseQuery.add(Term("source", "info"))
        hits = searcher.search(phraseQuery)
        self.assertEqual(2, len(hits))
    
        termQuery = TermQuery(Term("contents","foobar"))
        booleanQuery = BooleanQuery()
        booleanQuery.add(termQuery, BooleanClause.Occur.MUST)
        booleanQuery.add(phraseQuery, BooleanClause.Occur.MUST)
        hits = searcher.search(booleanQuery)
        self.assertEqual(1, len(hits))
    
        searcher.close()
    
        writer = IndexWriter(directory, WhitespaceAnalyzer(), True)
        doc = Document()
        doc.add(Field("contents", "map entry woo",
                      Field.Store.YES, Field.Index.TOKENIZED,
                      Field.TermVector.YES))
        writer.addDocument(doc)

        doc = Document()
        doc.add(Field("contents", "woo map entry",
                      Field.Store.YES, Field.Index.TOKENIZED,
                      Field.TermVector.YES))
        writer.addDocument(doc)

        doc = Document()
        doc.add(Field("contents", "map foobarword entry woo",
                      Field.Store.YES, Field.Index.TOKENIZED,
                      Field.TermVector.YES))
        writer.addDocument(doc)

        writer.optimize()
        writer.close()
    
        searcher = IndexSearcher(directory)
    
        termQuery = TermQuery(Term("contents", "woo"))
        phraseQuery = PhraseQuery()
        phraseQuery.add(Term("contents", "map"))
        phraseQuery.add(Term("contents", "entry"))
    
        hits = searcher.search(termQuery)
        self.assertEqual(3, len(hits))
        hits = searcher.search(phraseQuery)
        self.assertEqual(2, len(hits))
    
        booleanQuery = BooleanQuery()
        booleanQuery.add(termQuery, BooleanClause.Occur.MUST)
        booleanQuery.add(phraseQuery, BooleanClause.Occur.MUST)
        hits = searcher.search(booleanQuery)
        self.assertEqual(2, len(hits))
    
        booleanQuery = BooleanQuery()
        booleanQuery.add(phraseQuery, BooleanClause.Occur.MUST)
        booleanQuery.add(termQuery, BooleanClause.Occur.MUST)
        hits = searcher.search(booleanQuery)
        self.assertEqual(2, len(hits))
    
        searcher.close()
        directory.close()


if __name__ == "__main__":
    import sys
    if '-loop' in sys.argv:
        sys.argv.remove('-loop')
        while True:
            try:
                main()
            except:
                pass
    else:
         main()



#!/usr/bin/env python
from PyLucene import QueryParser, IndexSearcher, StandardAnalyzer, FSDirectory
from PyLucene import VERSION, LUCENE_VERSION

"""
This script is loosely based on the Lucene (java implementation) demo class 
org.apache.lucene.demo.SearchFiles.  It will prompt for a search query, then it
will search the Lucene index in the current directory called 'index' for the
search query entered against the 'contents' field.  It will then display the
'path' and 'name' fields for each of the hits it finds in the index.  Note that
search.close() is currently commented out because it causes a stack overflow in
some cases.
"""
def run(searcher, analyzer):
    while True:
        print
        print "Hit enter with no input to quit."
        command = raw_input("Query:")
        if command == '':
            return

        print
        print "Searching for:", command
        query = QueryParser("contents", analyzer).parse(command)
        hits = searcher.search(query)
        print "%s total matching documents." % hits.length()
        for i, doc in hits:
            print 'path:', doc.get("path"), 'name:', doc.get("name")


if __name__ == '__main__':
    STORE_DIR = "index"
    print 'PyLucene', VERSION, 'Lucene', LUCENE_VERSION
    directory = FSDirectory.getDirectory(STORE_DIR, False)
    searcher = IndexSearcher(directory)
    analyzer = StandardAnalyzer()
    run(searcher, analyzer)
    searcher.close()




import os, sys, unittest

sys.path.append(os.path.dirname(os.path.abspath(sys.argv[0])))

import lia.indexing.CompoundVersusMultiFileIndexTest
unittest.main(lia.indexing.CompoundVersusMultiFileIndexTest)



# ====================================================================
# Copyright (c) 2004-2005 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions: 
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software. 
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#

from unittest import TestCase
from PyLucene import StopAnalyzer

from lia.analysis.AnalyzerUtils import AnalyzerUtils
from lia.analysis.stopanalyzer.StopAnalyzerFlawed import StopAnalyzerFlawed
from lia.analysis.stopanalyzer.StopAnalyzer2 import StopAnalyzer2


class StopAnalyzerAlternativeTest(TestCase):

    def testStopAnalyzer2(self):

        tokens = AnalyzerUtils.tokensFromAnalysis(StopAnalyzer2(),
                                                  "The quick brown...")
        AnalyzerUtils.assertTokensEqual(self, tokens, ["quick", "brown"])

    def testStopAnalyzerFlawed(self):

        tokens = AnalyzerUtils.tokensFromAnalysis(StopAnalyzerFlawed(),
                                                  "The quick brown...")
        self.assertEqual("the", tokens[0].termText())


    #
    # Illustrates that "the" is not removed, although it is lowercased
    #

    def main(cls):

        AnalyzerUtils.displayTokens(StopAnalyzerFlawed(),
                                    "The quick brown...")

    main = classmethod(main)



# ====================================================================
# Copyright (c) 2004-2005 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions: 
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software. 
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#

from lia.common.LiaTestCase import LiaTestCase

from PyLucene import \
    SimpleAnalyzer, MultiFieldQueryParser, IndexSearcher, BooleanClause


class MultiFieldQueryParserTest(LiaTestCase):

    def testDefaultOperator(self):

        SHOULD = BooleanClause.Occur.SHOULD
        query = MultiFieldQueryParser.parseQueries("development",
                                                   ["title", "subject"],
                                                   [SHOULD, SHOULD],
                                                   SimpleAnalyzer())

        searcher = IndexSearcher(self.directory)
        hits = searcher.search(query)

        self.assertHitsIncludeTitle(hits, "Java Development with Ant")

        # has "development" in the subject field
        self.assertHitsIncludeTitle(hits, "Extreme Programming Explained")

    def testSpecifiedOperator(self):
        
        MUST = BooleanClause.Occur.MUST
        query = MultiFieldQueryParser.parseQueries("development",
                                                   ["title", "subject"],
                                                   [MUST, MUST],
                                                   SimpleAnalyzer())

        searcher = IndexSearcher(self.directory)
        hits = searcher.search(query)

        self.assertHitsIncludeTitle(hits, "Java Development with Ant")
        self.assertEqual(1, len(hits), "one and only one")



# ====================================================================
# Copyright (c) 2004-2005 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions: 
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software. 
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#

from PyLucene import \
     WhitespaceAnalyzer, Document, Field, IndexReader, IndexWriter


class T9er(object):

    keys = [         "2abc", "3def",
            "4ghi",  "5jkl", "6mno",
            "7pqrs", "8tuv", "9wxyz"]

    keyMap = {}

    def main(cls, argv):
        
        if len(argv) != 3:
            print "Usage: T9er <WordNet index dir> <t9 index>"
            return
        
        for key in cls.keys:
            c = key[0]
            k = key[1:]
            for kc in k:
                cls.keyMap[kc] = c
                print kc, "=", c

        indexDir = argv[1]
        t9dir = argv[2]

        reader = IndexReader.open(indexDir)

        numDocs = reader.maxDoc()
        print "Processing", numDocs, "words"

        writer = IndexWriter(t9dir, WhitespaceAnalyzer(), True)

        for id, origDoc in reader:
            word = origDoc.get("word")
            if word is None or len(word) == 0:
                continue

            newDoc = Document()
            newDoc.add(Field("word", word,
                             Field.Store.YES, Field.Index.UN_TOKENIZED))
            newDoc.add(Field("t9", cls.t9(word),
                             Field.Store.YES, Field.Index.UN_TOKENIZED))
            newDoc.add(Field("length", str(len(word)),
                             Field.Store.NO, Field.Index.UN_TOKENIZED))
            writer.addDocument(newDoc)
            if id % 100 == 0:
                print "Document", id

        writer.optimize()
        writer.close()

        reader.close()

    def t9(cls, word):

        return ''.join([cls.keyMap[c] for c in word])

    main = classmethod(main)
    t9 = classmethod(t9)





# Prints the filename and size of each file within a given compound file.
# Add the -extract flag to extract files to the current working directory.
# In order to make the extracted version of the index work, you have to copy
# the segments file from the compound index into the directory where the
# extracted files are stored.

import sys
from PyLucene import IndexReader

if len(sys.argv) < 2:
    print 'Usage: python index_reader.py [-extract] cfsfile'
else:
    IndexReader.main(sys.argv[1:])



# ====================================================================
# Copyright (c) 2004-2005 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions: 
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software. 
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#

from unittest import TestCase, main
from PyLucene import *


class FilteredQueryTestCase(TestCase):
    """
    Unit tests ported from Java Lucene
    """

    def setUp(self):

        self.directory = RAMDirectory()
        writer = IndexWriter(self.directory, WhitespaceAnalyzer(), True)

        doc = Document()
        doc.add(Field("field", "one two three four five",
                      Field.Store.YES, Field.Index.TOKENIZED))
        doc.add(Field("sorter", "b",
                      Field.Store.YES, Field.Index.TOKENIZED))
                      
        writer.addDocument(doc)

        doc = Document()
        doc.add(Field("field", "one two three four",
                      Field.Store.YES, Field.Index.TOKENIZED))
        doc.add(Field("sorter", "d",
                      Field.Store.YES, Field.Index.TOKENIZED))

        writer.addDocument(doc)

        doc = Document()
        doc.add(Field("field", "one two three y",
                      Field.Store.YES, Field.Index.TOKENIZED))
        doc.add(Field("sorter", "a",
                      Field.Store.YES, Field.Index.TOKENIZED))

        writer.addDocument(doc)

        doc = Document()
        doc.add(Field("field", "one two x",
                      Field.Store.YES, Field.Index.TOKENIZED))
        doc.add(Field("sorter", "c",
                      Field.Store.YES, Field.Index.TOKENIZED))
                      
        writer.addDocument(doc)

        writer.optimize()
        writer.close()

        self.searcher = IndexSearcher(self.directory)
        self.query = TermQuery(Term("field", "three"))

        class filter(object):
            def bits(self, reader):
                bitset = BitSet(5)
                bitset.set(1)
                bitset.set(3)
                return bitset

        self.filter = filter()

    def tearDown(self):

        self.searcher.close()
        self.directory.close()

    def testFilteredQuery(self):

        filteredquery = FilteredQuery(self.query, self.filter)
        hits = self.searcher.search(filteredquery);
        self.assertEqual(1, hits.length())
        self.assertEqual(1, hits.id(0))

        hits = self.searcher.search(filteredquery, Sort("sorter"))
        self.assertEqual(1, hits.length())
        self.assertEqual(1, hits.id(0))

        filteredquery = FilteredQuery(TermQuery(Term("field", "one")),
                                      self.filter)
        hits = self.searcher.search(filteredquery)
        self.assertEqual(2, hits.length())

        filteredquery = FilteredQuery(TermQuery(Term("field", "x")),
                                      self.filter)
        hits = self.searcher.search(filteredquery)
        self.assertEqual(1, hits.length())
        self.assertEqual(3, hits.id(0))

        filteredquery = FilteredQuery(TermQuery(Term("field", "y")),
                                      self.filter)
        hits = self.searcher.search(filteredquery)
        self.assertEqual(0, hits.length())

    def testRangeQuery(self):
        """
        This tests FilteredQuery's rewrite correctness
        """
        
        rq = RangeQuery(Term("sorter", "b"), Term("sorter", "d"), True)
        filteredquery = FilteredQuery(rq, self.filter)
        hits = self.searcher.search(filteredquery)
        self.assertEqual(2, hits.length())


if __name__ == "__main__":
    import sys
    if '-loop' in sys.argv:
        sys.argv.remove('-loop')
        while True:
            try:
                main()
            except:
                pass
    else:
         main()




#
# Ported from LuceneBenchmark.java, which was written by Andrzej Bialecki
#
#  Preliminary Release Notes
#  -------------------------
#
#  This port doesn't seem to run to completion when
#  setUseCompoundFile(False) is called on the IndexWriter. Without compound
#  files enabled, it fails with a 'too many open files' error. To reproduce
#  the error, look for 'for compound in LuceneBenchmark.bools[n:]:' and
#  change the previous line to say 'n = 0'.
# 
#  Some searches return no hits, the logic had to be changed to accomodate
#  for that to avoid devide-by-zero errors. Maybe IndexWriter.maxFieldLength
#  needs to be set ? Maybe there is a bug ? Maybe the set of documents was
#  too small during testing ?
#
#  The resulting performance numbers were not compared with the Java
#  version or validated for accuracy.
#
#  To lengthen the benchmark, switch to a larger document source by
#  changing the sources attribute of the LuceneBenchmark class below.
# 
#  You may also increase RUN_COUNT or SCALE_UP, or increase the jumbo files.
#  Look for 'for k in xrange(11, 21):' and increase the second number.
#

import os, sys, tarfile, time

from cStringIO import StringIO
from urlparse import urlparse
from urllib import urlretrieve

from Streams import InputStreamReader, StringReader

from PyLucene import Analyzer, StandardAnalyzer, DateField, Document, Field
from PyLucene import IndexReader, IndexWriter, IndexSearcher
from PyLucene import QueryParser, Hits, Query, Directory, FSDirectory
from PyLucene import System, Runtime, DateFormat, Date


class LuceneBenchmark(object):
    
    bools = [ False, True ]
    queries = [ "file:src", "body:article", "body:article subject:re",
                "file:comp body:article* subject:re" ]
  
    # Number of iterations for each operation.
    RUN_COUNT = 1

    # Artificially scale up the number of source documents by this factor.
    SCALE_UP = 1

    LOG_STEP = 250
    SETUP_DONE = ".setup_done"

    # Default constructor - initialize the a list of URLs with source corpora.

    def __init__(self):
        
        self.workDir = None
        self.indexDir = None
        self.srcUrl = None
        self.sourceDir = None
        self.jumboDir = None
        self.setup_done = None

#        self.sources = [ "http://www-2.cs.cmu.edu/afs/cs.cmu.edu/project/theo-20/www/data/news20.tar.gz", "http://people.csail.mit.edu/u/j/jrennie/public_html/20Newsgroups/20news-18828.tar.gz", "http://kdd.ics.uci.edu/databases/20newsgroups/mini_newsgroups.tar.gz" ]

        self.sources = [ "http://kdd.ics.uci.edu/databases/20newsgroups/mini_newsgroups.tar.gz" ]
  
    def setWorkDir(self, wd):
        
        if self.setup_done is not None:
            raise RuntimeError, "Too late - setup already done."

        self.workDir = wd

    # Download a source file from URL.
    # return local file, or None if failed

    def getSourceFile(self):

        self.pct = 0
        
        def hook(blockCount, blockSize, totalSize):
            if blockCount == 0:
                print '    reading', totalSize, 'bytes',
                sys.stdout.flush()
            elif totalSize > 0:
                if (float(blockCount *
                          blockSize) / totalSize) * 100 > self.pct + 10:
                    sys.stdout.write('.')
                    sys.stdout.flush()
                    self.pct += 10

        for source in self.sources:
            try:
                scheme, host, pathname, x, y, z = urlparse(source)
                filename = os.path.basename(pathname)
                print '    retrieving', source, 'to', filename
                filename, headers = urlretrieve(source, filename, hook)
                sys.stdout.write('\n')
                return filename
            except Exception, e:
                print e
                print '    failed'
                continue

    # Delete files and directories, even if non-empty.
    #  param dir file or directory
    #  return True on success, False if no or part of files have been deleted

    def fullyDelete(self, dir):

        if dir is None or not os.path.exists(dir):
            return False
        
        def purge(arg, path, names):
            for name in names:
                name = os.path.join(path, name)
                if os.path.isdir(name):
                    os.path.walk(name, purge, None)
                    os.rmdir(name)
                else:
                    os.remove(name)

        os.path.walk(dir, purge, None)
        os.rmdir(dir)

        return True
      
    # Make sure the sources are downloaded and unpacked, remove old indexes.
    # Prepare a set of large documents.

    def setup(self):

        if self.workDir is None:
            self.workDir = os.tempnam(None, ".lucene_benchmark")
            os.makedirs(self.workDir)

        print " - setup in", os.path.abspath(self.workDir)
        os.chdir(self.workDir)

        # reuse old setup
        self.setup_done = os.path.join(self.workDir, ".setup_done")
        self.indexDir = os.path.join(self.workDir, "index")
        self.sourceDir = os.path.join(self.workDir, "src")
        self.jumboDir = os.path.join(self.workDir, "jumbo", "jumbo")

        self.reset()
        if os.path.exists(self.setup_done):
            return

        src = None
        # check if one of the sources is downloaded
        for source in self.sources:
            scheme, host, pathname, x, y, z = urlparse(source)
            filename = os.path.basename(pathname)
            if os.path.exists(filename):
                src = filename
                break

        if src is None:
            src = self.getSourceFile()

        self.fullyDelete(self.sourceDir)
        os.makedirs(self.sourceDir)

        print "* Unpacking reference collection:", src,
        dircnt = 0
        fcnt = 0
        tar = tarfile.open(src, "r:gz")

        while True:
            member = tar.next()
            if member is None:
                break
            if fcnt % 100 == 0:
                sys.stdout.write('+')
                sys.stdout.flush()
            tar.extract(member, self.sourceDir)
            if member.isfile():
                fcnt += 1
            elif member.isdir():
                dircnt += 1

        tar.close()
        sys.stdout.write('\n')
        print " -", fcnt, "source files in", dircnt, "directories."
        print "* Creating jumbo files..."

        fcnt = 0
        dircnt = 0

        # concatenate
        self.fullyDelete(self.jumboDir)
        os.makedirs(self.jumboDir)

        groupDir = os.path.join(self.sourceDir, os.listdir(self.sourceDir)[0])
        groups = os.listdir(groupDir)
        for group in groups:
            outdir = os.path.join(self.jumboDir, group)
            os.makedirs(outdir)
            dircnt += 1
            print " - creating jumbo files in", outdir

            fileDir = os.path.join(groupDir, group)
            files = os.listdir(fileDir)
            for k in xrange(11, 21):
                output = file(os.path.join(outdir, str(k)), "w")
                for m in xrange(0, k):
                    input = file(os.path.join(fileDir, files[m]))
                    data = input.read()
                    input.close()
                    output.write(data)
                    fcnt += 1
                output.close()
        print " -", fcnt, "jumbo files in", dircnt, "directories."

        # create the "done" file
        file(self.setup_done, "w").close()

    # Remove existing index.

    def reset(self):

        if os.path.exists(self.indexDir):
            self.fullyDelete(self.indexDir)
        os.makedirs(self.indexDir)

    # Remove index and unpacked source files. You have to run setup() after
    # you run this method.

    def clean(self):

        self.reset()
        self.fullyDelete(self.jumboDir)
        self.fullyDelete(self.sourceDir)
        os.remove(self.setup_done)

    # Assume the input is an NNTP message, where the header is separated from
    # the body with blank line. <br>
    # Extract basic metadata from the header: "From:", "Subject:", "Date:". If
    # more fields are needed, the first couple of lines from the body will be
    # converted into additional fields. <br>
    # NOTE: this method doesn't even pretend to be an RFC-compliant parser, so
    # don't expect any MIME or transport decoding or similar.
    # 
    #  param in input file
    #  param addFields if greater than 0, add more fields named "line0",
    #        "line1", "line2", etc, with the content made from the body text.
    #  return Lucene document

    def makeDocument(self, filename, addFields, tags, stored, tokenized, tfv):

        doc = Document()

        # tag this document
        if tags:
            i = 0
            for tag in tags:
                doc.add(Field("%s%d" %(tag, i), tag,
                              stored, True, tokenized, tfv))
                i += 1

        doc.add(Field("file", os.path.abspath(filename),
                      stored, True, tokenized, tfv))

        header = []
        body = []

        input = file(filename)
        reader = InputStreamReader(input, 'iso-8859-1')
        
        inHeader = True
        while True:
            line = reader.readline()
            if line == '':
                break
            if inHeader:
                if line.strip() == '':
                    inHeader = False
                    continue
                header.append(line)
            else:
                body.append(line)

        reader.close()

        for line in header:
            if line.startswith("From: "):
                doc.add(Field("from", line[6:],
                              stored, True, tokenized, tfv))
            elif line.startswith("Subject: "):
                doc.add(Field("subject", line[9:],
                              stored, True, tokenized, tfv))
            elif line.startswith("Date: "):
                # parse date
                val = None
                date = line[6:].strip()
                if ',' in date:
                    format = '%a, %d %b %Y %H:%M:%S %Z'
                else:
                    format = '%a %d %b %Y %H:%M:%S %Z'
                try:
                    t = long(time.mktime(time.strptime(date, format))) * 1000
                    val = DateField.timeToString(t)
                except:
                    # print " -", filename, ": bad date '%s'" %(date)
                    val = DateField.timeToString(System.currentTimeMillis())
                doc.add(Field("date", val, stored, True, False, False))

        # if additional fields are needed, add them here
        for i in xrange(addFields, len(body)):
            doc.add(Field("line%d" %(i), body[i], stored, True, tokenized, tfv))
        
        # add body
        doc.add(Field("body", u'\n'.join(body), stored, True, tokenized, tfv))

        return doc

    # Make index, and collect time data.
    #  param trd run data to populate
    #  param srcDir directory with source files
    #  param iw index writer, already open
    #  param addFields fields to add to each document (see makeDocument method)
    #  param stored store values of fields
    #  param tokenized tokenize fields
    #  param tfv store term vectors

    def makeIndex(self, trd, srcDir, iw, addFields, stored, tokenized, tfv):

        groupDir = os.path.join(srcDir, os.listdir(srcDir)[0])
        groups = os.listdir(groupDir)

        doc = None
        cnt = 0L

        td = TimeData()
        td.name = "addDocument"
        
        for s in xrange(0, self.SCALE_UP):
            tags = [ "%s/%d" %(srcDir, s) ]
            for group in groups:
                fileDir = os.path.join(groupDir, group)
                files = os.listdir(fileDir)
                for file in files:
                    doc = self.makeDocument(os.path.join(fileDir, file),
                                            addFields, tags,
                                            stored, tokenized, tfv)
                    td.start()
                    iw.addDocument(doc)
                    td.stop()
                    cnt += 1
                    if cnt % self.LOG_STEP == 0:
                        print " - processed", cnt, ", run id=", trd.id
                    trd.addData(td)
                    td.reset()

        trd.addData(td)

    # Run benchmark using supplied parameters.
    #  param params benchmark parameters

    def runBenchmark(self, params):
        
        for i in xrange(0, self.RUN_COUNT):
            trd = TestRunData()
            trd.startRun()
            trd.id = str(i)
            iw = IndexWriter(params.dir, params.analyzer, True)
            iw.mergeFactor = params.mergeFactor
            iw.minMergeDocs = params.minMergeDocs
            iw.setUseCompoundFile(params.compound)
            self.makeIndex(trd, params.source, iw, 5, True, True, False)

            if params.optimize:
                td = TimeData("optimize")
                trd.addData(td)
                td.start()
                iw.optimize()
                td.stop()
                trd.addData(td)

            iw.close()

            if params.queries is not None:
                ir = None
                searcher = None
                for qd in params.queries:
                    if ir is not None and qd.reopen:
                        searcher.close()
                        ir.close()
                        ir = None
                        searcher = None

                    if ir is None:
                        ir = IndexReader.open(params.dir)
                        searcher = IndexSearcher(ir)

                    doc = None
                    if qd.warmup:
                        td = TimeData(qd.id + "-warm")
                        for m in xrange(0, ir.maxDoc()):
                            td.start()
                            if ir.isDeleted(m):
                                td.stop()
                                continue

                            doc = ir.document(m)
                            td.stop()

                        trd.addData(td)

                    td = TimeData(qd.id + "-srch")
                    td.start()
                    h = searcher.search(qd.q)
                    td.stop()
                    if h is not None and len(h) == 0:
                        print qd.q, 'returned no hits'

                    trd.addData(td)
                    td = TimeData(qd.id + "-trav")
                    if h is not None and len(h) > 0:
                        for m in xrange(0, len(h)):
                            td.start()
                            id = h.id(m)
                            if qd.retrieve:
                                doc = ir.document(id)
                            td.stop()

                    trd.addData(td)

                try:
                    if searcher is not None:
                        searcher.close()
                except:
                    pass
                
                try:
                    if ir is not None:
                        ir.close()
                except:
                    pass

            trd.endRun()
            params.runData.append(trd)

     # Optional argument points to the output directory for the test.
     #  param args

    def main(cls, args):
      
        bench = LuceneBenchmark()
        if args:
            bench.setWorkDir(args[0])
        bench.setup()

        a = StandardAnalyzer()
        qs = cls.createQueries(LuceneBenchmark.queries, a)

        # Here you can limit the set of query benchmarks
        qds = QueryData.getAll(qs)

        # Here you can narrow down the set of test parameters
        params = TestData.getAll([ bench.sourceDir, bench.jumboDir ],
                                 [ a ])

        for param in params:
            try:
                bench.reset()
                param.dir = FSDirectory.getDirectory(bench.indexDir, True)
                param.queries = qds
                print param
                bench.runBenchmark(param)

                # Here you can collect and output the runData for
                # further processing.
                print param.showRunData(param.id)

                #bench.runSearchBenchmark(queries, dir)
                param.dir.close()
                System.runFinalization()
                System.gc()
            except Exception, e:
                raise
                #print "EXCEPTION: ", e
                #break

    main = classmethod(main)

    # Parse the strings containing Lucene queries.
    #  param qs array of strings containing query expressions
    #  param a analyzer to use when parsing queries
    #  return array of Lucene queries

    def createQueries(cls, qs, a):

        qp = QueryParser("body", a)
        queries = []
        for _qs in qs:
            try:
                q = qp.parseQuery(_qs)
                queries.append(q)
            except Exception, e:
                raise
                #print e

        return queries

    createQueries = classmethod(createQueries)
    

# This class holds parameters for a query benchmark.

class QueryData(object):

    def __init__(self):

        # Benchmark id
        self.id = None

        # Lucene query
        self.q = None

        # If true, re-open index reader before benchmark.
        self.reopen = False
        
        # If true, warm-up the index reader before searching by sequentially
        # retrieving all documents from index.
        self.warmup = False

        # If true, actually retrieve documents returned in Hits.
        self.retrieve = False
  
    # Prepare a list of benchmark data, using all possible combinations of
    # benchmark parameters.
    #  param queries source Lucene queries

    def getAll(cls, queries):

        vqd = []
        i = 0
        for query in queries:
            for r in xrange(1, -1, -1):
                for w in xrange(1, -1, -1):
                    for t in xrange(0, 2):
                        qd = QueryData()
                        qd.id = "qd-%d%d%d%d" %(i, r, w, t)
                        qd.reopen = LuceneBenchmark.bools[r]
                        qd.warmup = LuceneBenchmark.bools[w]
                        qd.retrieve = LuceneBenchmark.bools[t]
                        qd.q = queries[i]

                        vqd.append(qd)
            i += 1

        return vqd

    getAll = classmethod(getAll)
  
    # Short legend for interpreting toString() output. */
    def getLabels(cls):
        return "# Query data: R-reopen, W-warmup, T-retrieve, N-no"

    getLabels = classmethod(getLabels)

    def __str__(self):
        return "id %s %s %s [%s]" %(self.reopen and "R" or "NR",
                                    self.warmup and "W" or "NW",
                                    self.retrieve and "T" or "NT", self.q)


# This class holds a data point measuring speed of processing.

class TimeData(object):

    def __init__(self, name=None):
        
        # Name of the data point - usually one of a data series
        # with the same name
        self.name = name
    
        # Number of records processed.
        self.count = 0

        # Elapsed time in floating point seconds.
        self.elapsed = 0.0
  
        self.delta = 0.0

        # Free memory at the end of measurement interval.
        self.freeMem = 0L

        # Total memory at the end of measurement interval.
        self.totalMem = 0L

    # Start counting elapsed time.

    def start(self):
        self.delta = time.time()

    # Stop counting elapsed time.

    def stop(self):

        self.count += 1
        self.elapsed += time.time() - self.delta

    # Record memory usage.

    def recordMemUsage(self):

        runtime = Runtime.getRuntime()
        self.freeMem = runtime.freeMemory()
        self.totalMem = runtime.totalMemory()


    # Reset counters.

    def reset(self):

        self.count = 0
        self.elapsed = 0L
        self.delta = 0L

    def clone(self):

        td = TimeData(self.name)
        td.elapsed = self.elapsed
        td.count = self.count
        td.delta = self.delta
        td.freeMem = self.freeMem
        td.totalMem = self.totalMem

        return td
  
    # Get rate of processing, defined as number of processed records per second.

    def getRate(self):

        try:
            return self.count / self.elapsed
        except ZeroDivisionError:
            print self.name, 'count:', self.count, 'elapsed:', self.elapsed
            return 0

    # Get a short legend for toString() output.

    def getLabels(cls):
        return "# count\telapsed\trec/s\tfreeMem\ttotalMem"

    getLabels = classmethod(getLabels)
  
    # Return a tab-seprated string containing this data.
    #  param withMem if true, append also memory information

    def __str__(self, withMem=None):

        if withMem is not None:
            s = "%d\t%d\t%f\t" %(self.count, self.elapsed, self.getRate())
            if withMem == True:
                s += "\t%d\t%d" %(self.freeMem, self.totalMem)
            return s
        
        return 'True'

# This class holds series of TimeData related to a single test run. TimeData
# values may contribute to different measurements, so this class provides also
# some useful methods to separate them.

class TestRunData(object):

    def __init__(self, id=None):
        
        self.id = id

        # Start and end time of this test run.
        self.start = 0L
        self.end = 0L

        self.data = {}

    # Mark the starting time of this test run.
    def startRun(self):
        self.start = time.time()

    # Mark the ending time of this test run.
    def endRun(self):
        self.end = time.time()

    # Add a data point.
    def addData(self, td):

        td.recordMemUsage()
        try:
            v = self.data[td.name]
        except KeyError:
            v = []
            self.data[td.name] = v

        v.append(td.clone())

    # Get a list of all available types of data points.
    def getLabels(self):
        return self.data.iterkeys()

    # Get total values from all data points of a given type.

    def getTotal(self, label):

        try:
            v = self.data[label]
        except KeyError:
            return None

        res = TimeData("TOTAL " + label)
        for td in v:
            res.count += td.count
            res.elapsed += td.elapsed

        return res

    # Get total values from all data points of all types.
    #  return a list of TimeData values for all types.
    def getTotals(self):
        return [ self.getTotal(label) for label in self.getLabels() ]

    # Get memory usage stats. for a given data type.

    def getMemUsage(self, label):

        try:
            v = self.data[label]
        except KeyError:
            return None

        res = MemUsage()
        res.minFree = (1L << 63) - 1
        res.minTotal = (1L << 63) - 1
        avgFree = 0L
        avgTotal = 0L
        for td in v:
            if res.maxFree < td.freeMem:
                res.maxFree = td.freeMem
            if res.maxTotal < td.totalMem:
                res.maxTotal = td.totalMem
            if res.minFree > td.freeMem:
                res.minFree = td.freeMem
            if res.minTotal > td.totalMem:
                res.minTotal = td.totalMem
            avgFree += td.freeMem
            avgTotal += td.totalMem

        res.avgFree = avgFree / len(v)
        res.avgTotal = avgTotal / len(v)

        return res

    def __str__(self):

        return '\n'.join(["%s-%s %s %s" %(self.id, label,
                                          self.getTotal(label).__str__(False),
                                          self.getMemUsage(label).toScaledString(1024 * 1024, "MB"))
                          for label in self.getLabels()])

# This class holds a set of memory usage values.
class MemUsage(object):

    def __init__(self):

        self.maxFree = 0L
        self.minFree = 0L
        self.avgFree = 0L

        self.maxTotal = 0L
        self.minTotal = 0L
        self.avgTotal = 0L

    def __str__(self):
        selftoScaledString(1, "B")

    # Scale down the values by divisor, append the unit string.
    
    def toScaledString(self, div, unit):

        return "free=%d/%d/%d %s, total=%d/%d/%d %s" %(self.minFree / div,
                                                       self.avgFree / div,
                                                       self.maxFree / div,
                                                       unit,
                                                       self.minTotal / div,
                                                       self.avgTotal / div,
                                                       self.maxTotal / div,
                                                       unit)

# This class holds together all parameters related to a test. Single test is
# performed several times, and all results are averaged.

class TestData(object):

    MINMERGE_COUNTS = [ 10, 20, 50, 100, 200, 500 ]
    MERGEFACTOR_COUNTS = [ 10, 20, 50, 100, 200, 500 ]

    def __init__(self):
        
        # ID of this test data.
        self.id = None

        # Heap size.
        self.heap = Runtime.getRuntime().maxMemory()

        # List of results for each test run with these parameters.
        self.runData = []

        self.minMergeDocs = 0
        self.mergeFactor = 0
        
        # Directory containing source files.
        self.source = None

        # Lucene Directory implementation for creating an index.
        self.dir = None

        # Analyzer to use when adding documents.

        self.analyzer = None

        # If true, use compound file format.
        self.compound = False
  
        # If true, optimize index when finished adding documents.
        self.optimize = False

        # Data for search benchmarks.
        self.queries = None

    # Get a textual summary of the benchmark results, average from test runs.

    def showRunData(self, prefix):

        class DCounter(object):
            def __init__(self):
                self.total = 0
                self.count = 0
                self.recordCount = 0
    
        class LCounter(object):
            def __init__(self):
                self.total = 0
                self.count = 0
  
        if len(self.runData) == 0:
            return "# [NO RUN DATA]"

        sb = StringIO()
        sb.write("# testData id\toperation\trunCnt\trecCnt\trec/s\tavgFreeMem\tavgTotalMem\n")

        mapMem = {}
        mapSpeed = {}

        for trd in self.runData:
            for label in trd.getLabels():
                mem = trd.getMemUsage(label)
                if mem is not None:
                    try:
                        tm = mapMem[label]
                    except KeyError:
                        tm = [ LCounter(), LCounter() ]
                        mapMem[label] = tm
                    tm[0].total += mem.avgFree
                    tm[0].count += 1
                    tm[1].total += mem.avgTotal
                    tm[1].count += 1
        
                td = trd.getTotal(label)
                if td is not None:
                    try:
                        dc = mapSpeed[label]
                    except KeyError:
                        dc = DCounter()
                        mapSpeed[label] = dc
                    dc.count += 1
                    dc.total += td.getRate()
                    dc.recordCount += td.count

        res = {}
        for label, dc in mapSpeed.iteritems():
            res[label] = "%d\t%d\t%f" %(dc.count,
                                        dc.recordCount / dc.count,
                                        dc.total / float(dc.count))

        for label, lc in mapMem.iteritems():
            try:
                speed = res[label]
            except KeyError:
                speed = "%d\0.0" %(lc[0].count)

            res[label] = "%s\t%s\t%s" %(speed,
                                        lc[0].total / lc[0].count,
                                        lc[1].total / lc[1].count)

        for label, r in res.iteritems():
            sb.write("%s\t%s\t%s\n" %(prefix, label, r))

        value = sb.getvalue()
        sb.close()

        return value
  
    # Prepare a list of benchmark data, using all possible combinations of
    # benchmark parameters.
    #  param sources list of directories containing different source document
    #  collections
    #  param list of analyzers to use.

    def getAll(cls, sources, analyzers):

        res = []
        ref = TestData()

        q = 0
        for analyzer in analyzers:
            m = 0
            for source in sources:
                i = 0
                for minMergeDocs in TestData.MINMERGE_COUNTS:
                    k = 0
                    for mergeFactor in TestData.MERGEFACTOR_COUNTS:
                        n = 1
                        for compound in LuceneBenchmark.bools[n:]:
                            p = 0
                            for optimize in LuceneBenchmark.bools:
                                ref.id = "td-%d%d%d%d%d%d" %(q, m, i, k, n, p)
                                ref.source = source
                                ref.analyzer = analyzer
                                ref.minMergeDocs = minMergeDocs
                                ref.mergeFactor = mergeFactor
                                ref.compound = compound
                                ref.optimize = optimize
                                res.append(ref.clone())
                                p += 1
                            n += 1
                        k += 1
                    i += 1
                m += 1
            q += 1

        return res

    getAll = classmethod(getAll)

    def clone(self):

        cl = TestData()
        cl.id = self.id
        cl.compound = self.compound
        cl.heap = self.heap
        cl.mergeFactor = self.mergeFactor
        cl.minMergeDocs = self.minMergeDocs
        cl.optimize = self.optimize
        cl.source = self.source
        cl.dir = self.dir
        cl.analyzer = self.analyzer
        # don't clone runData
        return cl

    def __str__(self):

        res = StringIO()
        res.write("#-- ID: %s, %s heap=%d --\n" %(self.id,
                                                  DateFormat.getInstance().format(Date()),
                                                  self.heap))
        res.write("# source=%s, dir=%s\n" %(self.source, self.dir))
        res.write("# minMergeDocs=%d, mergeFactor=%d" %(self.minMergeDocs,
                                                        self.mergeFactor))
        res.write(", compound=%d, optimize=%d\n" %(self.compound,
                                                   self.optimize))
        if self.queries is not None:
            res.write(QueryData.getLabels())
            res.write('\n')
            for qd in self.queries:
                res.write("# %s\n" %(qd))

        value = res.getvalue()
        res.close()
        
        return value


if __name__ == "__main__":
    LuceneBenchmark.main(sys.argv[1:])



# ====================================================================
# Copyright (c) 2004-2005 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions: 
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software. 
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#


from lia.analysis.AnalyzerUtils import AnalyzerUtils
from PyLucene import \
     StopAnalyzer, SimpleAnalyzer, WhitespaceAnalyzer, StandardAnalyzer


class AnalyzerDemo(object):

    examples = ["The quick brown fox jumped over the lazy dogs",
                "XY&Z Corporation - xyz@example.com"]
    
    analyzers = [WhitespaceAnalyzer(),
                 SimpleAnalyzer(),
                 StopAnalyzer(),
                 StandardAnalyzer()]

    def main(cls, argv):

        # Use the embedded example strings, unless
        # command line arguments are specified, then use those.
        strings = cls.examples

        if len(argv) > 1:
            strings = argv[1:]

        for string in strings:
            cls.analyze(string)

    def analyze(cls, text):

        print'"Analyzing "', text, '"'

        for analyzer in cls.analyzers:
            name = type(analyzer).__name__
            print " %s:" %(name),
            AnalyzerUtils.displayTokens(analyzer, text)
            print ''
        print ''

    main = classmethod(main)
    analyze = classmethod(analyze)


if __name__ == "__main__":
    import sys
    AnalyzerDemo.main(sys.argv)



# ====================================================================
# Copyright (c) 2004-2005 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions: 
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software. 
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#

import os

from bsddb.db import DBEnv, DB
from bsddb.db import \
     DB_INIT_MPOOL, DB_INIT_LOCK, DB_INIT_TXN, DB_THREAD, DB_BTREE

# missing from python interface at the moment
DB_LOG_INMEMORY = 0x00020000

from PyLucene import DbDirectory, IndexSearcher, Term, TermQuery


class BerkeleyDbSearcher(object):

    def main(cls, argv):

        if len(argv) != 2:
            print "Usage: BerkeleyDbSearcher <index dir>"
            return

        dbHome = argv[1]

        env = DBEnv()
        env.set_flags(DB_LOG_INMEMORY, 1);
        if os.name == 'nt':
            env.set_cachesize(0, 0x4000000, 1)
        elif os.name == 'posix':
            from commands import getstatusoutput
            if getstatusoutput('uname') == (0, 'Linux'):
                env.set_cachesize(0, 0x4000000, 1)

        env.open(dbHome, (DB_THREAD |
                          DB_INIT_MPOOL | DB_INIT_LOCK | DB_INIT_TXN), 0)

        index = DB(env)
        blocks = DB(env)
        txn = None

        try:
            txn = env.txn_begin(None)
            index.open(filename = '__index__', dbtype = DB_BTREE,
                       flags = DB_THREAD, txn = txn)
            blocks.open(filename = '__blocks__', dbtype = DB_BTREE,
                        flags = DB_THREAD, txn = txn)
        except:
            if txn is not None:
                txn.abort()
                txn = None
            raise
        else:
            txn.commit()
            txn = None

        try:
            txn = env.txn_begin(None)
            directory = DbDirectory(txn, index, blocks, 0)
            searcher = IndexSearcher(directory)

            hits = searcher.search(TermQuery(Term("contents", "fox")))
            print len(hits), "document(s) found"
            searcher.close()
        except:
            if txn is not None:
                txn.abort()
                txn = None
            raise
        else:
            txn.abort()

            index.close()
            blocks.close()
            env.close()

    main = classmethod(main)



# ====================================================================
# Copyright (c) 2004-2005 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions: 
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software. 
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#

from PyLucene import \
     IndexWriter, IndexReader, IndexSearcher, \
     WhitespaceAnalyzer, Document, Field, Term, TermQuery

from lia.indexing.BaseIndexingTestCase import BaseIndexingTestCase


class DocumentUpdateTest(BaseIndexingTestCase):

    def testUpdate(self):

        self.assertEqual(1, self.getHitCount("city", "Amsterdam"))

        reader = IndexReader.open(self.dir)
        reader.deleteDocuments(Term("city", "Amsterdam"))
        reader.close()

        writer = IndexWriter(self.dir, self.getAnalyzer(), False)
        doc = Document()
        doc.add(Field("id", "1", Field.Store.YES, Field.Index.UN_TOKENIZED))
        doc.add(Field("country", "Russia",
                      Field.Store.YES, Field.Index.NO))
        doc.add(Field("contents", "St. Petersburg has lots of bridges",
                      Field.Store.NO, Field.Index.TOKENIZED))
        doc.add(Field("city", "St. Petersburg",
                      Field.Store.YES, Field.Index.TOKENIZED))
        writer.addDocument(doc)
        writer.optimize()
        writer.close()

        self.assertEqual(0, self.getHitCount("city", "Amsterdam"))
        self.assertEqual(1, self.getHitCount("city", "Petersburg"))


    def getAnalyzer(self):

        return WhitespaceAnalyzer()


    def getHitCount(self, fieldName, searchString):

        searcher = IndexSearcher(self.dir)
        t = Term(fieldName, searchString)
        query = TermQuery(t)
        hits = searcher.search(query)
        hitCount = len(hits)
        searcher.close()

        return hitCount



# ====================================================================
# Copyright (c) 2004-2005 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions: 
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software. 
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#

from unittest import TestCase, main
from PyLucene import *


class PrefixQueryTestCase(TestCase):
    """
    Unit tests ported from Java Lucene
    """

    def _initializeIndex(self, values):

        writer = IndexWriter(self.dir, WhitespaceAnalyzer(), True)
        for value in values:
            self._insertDoc(writer, value)
        writer.close()

    def _insertDoc(self, writer, content):

        doc = Document()

        doc.add(Field("id", "id" + str(self.docCount),
                      Field.Store.YES, Field.Index.UN_TOKENIZED))
        doc.add(Field("content", content,
                      Field.Store.NO, Field.Index.TOKENIZED))

        writer.addDocument(doc)
        self.docCount += 1

    def _addDoc(self, content):

        writer = IndexWriter(self.dir, WhitespaceAnalyzer(), False)
        self._insertDoc(writer, content)
        writer.close()

    def setUp(self):

        self.docCount = 0
        self.dir = RAMDirectory()

    def testExclusive(self):

        query = RangeQuery(Term("content", "A"),
                           Term("content", "C"),
                           False)
        self._initializeIndex(["A", "B", "C", "D"])
        searcher = IndexSearcher(self.dir)
        hits = searcher.search(query)
        self.assertEqual(1, hits.length(),
                         "A,B,C,D, only B in range")
        searcher.close()

        self._initializeIndex(["A", "B", "D"])
        searcher = IndexSearcher(self.dir)
        hits = searcher.search(query)
        self.assertEqual(1, hits.length(),
                         "A,B,D, only B in range")
        searcher.close()

        self._addDoc("C")
        searcher = IndexSearcher(self.dir)
        hits = searcher.search(query)
        self.assertEqual(1, hits.length(),
                         "C added, still only B in range")
        searcher.close()

    def testInclusive(self):

        query = RangeQuery(Term("content", "A"),
                           Term("content", "C"),
                           True)

        self._initializeIndex(["A", "B", "C", "D"])
        searcher = IndexSearcher(self.dir)
        hits = searcher.search(query)
        self.assertEqual(3, hits.length(),
                         "A,B,C,D - A,B,C in range")
        searcher.close()

        self._initializeIndex(["A", "B", "D"])
        searcher = IndexSearcher(self.dir)
        hits = searcher.search(query)
        self.assertEqual(2, hits.length(),
                         "A,B,D - A and B in range")
        searcher.close()

        self._addDoc("C")
        searcher = IndexSearcher(self.dir)
        hits = searcher.search(query)
        self.assertEqual(3, hits.length(),
                         "C added - A, B, C in range")
        searcher.close()


if __name__ == "__main__":
    import sys
    if '-loop' in sys.argv:
        sys.argv.remove('-loop')
        while True:
            try:
                main()
            except:
                pass
    else:
         main()



# ====================================================================
# Copyright (c) 2004-2005 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions: 
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software. 
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#

import os

from PyLucene import \
     FSDirectory, Document, Field, IndexSearcher, SimpleAnalyzer, \
     RangeQuery, Sort, SortField, DecimalFormat, System, Term


class SortingExample(object):

    def __init__(self, directory):

        self.directory = directory

    def displayHits(self, query, sort):

        searcher = IndexSearcher(self.directory)
        hits = searcher.search(query, sort)

        print "\nResults for:", query, "sorted by", sort
        print "Title".rjust(30), "pubmonth".rjust(10), \
              "id".center(4), "score".center(15)

        scoreFormatter = DecimalFormat("0.######")
        for i, doc in hits:
            title = doc["title"]
            if len(title) > 30:
                title = title[:30]
            print title.encode('ascii', 'replace').rjust(30), \
                  doc["pubmonth"].rjust(10), \
                  str(hits.id(i)).center(4), \
                  scoreFormatter.format(hits.score(i)).ljust(12)
            print "  ", doc["category"]
            # print searcher.explain(query, hits.id(i))

        searcher.close()

    def main(cls, argv):

        earliest = Term("pubmonth", "190001")
        latest = Term("pubmonth", "201012")
        allBooks = RangeQuery(earliest, latest, True)

        indexDir = System.getProperty("index.dir")
        directory = FSDirectory.getDirectory(indexDir, False)
        example = SortingExample(directory)

        example.displayHits(allBooks, Sort.RELEVANCE)
        example.displayHits(allBooks, Sort.INDEXORDER)
        example.displayHits(allBooks, Sort("category"))
        example.displayHits(allBooks, Sort("pubmonth", True))

        example.displayHits(allBooks,
                            Sort([SortField("category"),
                                  SortField.FIELD_SCORE,
                                  SortField("pubmonth", SortField.INT, True)]))

        example.displayHits(allBooks,
                            Sort([SortField.FIELD_SCORE,
                                  SortField("category")]))

    main = classmethod(main)



# ====================================================================
# Copyright (c) 2004-2005 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions: 
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software. 
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#

from lia.common.LiaTestCase import LiaTestCase

from PyLucene import \
     SimpleAnalyzer, Document, TermQuery, QueryParser, IndexSearcher, Term


class BasicSearchingTest(LiaTestCase):

    def testTerm(self):

        searcher = IndexSearcher(self.directory)
        t = Term("subject", "ant")
        query = TermQuery(t)
        hits = searcher.search(query)
        self.assertEqual(1, hits.length(), "JDwA")

        t = Term("subject", "junit")
        hits = searcher.search(TermQuery(t))
        self.assertEqual(2, hits.length())

        searcher.close()

    def testKeyword(self):

        searcher = IndexSearcher(self.directory)
        t = Term("isbn", "1930110995")
        query = TermQuery(t)
        hits = searcher.search(query)
        self.assertEqual(1, hits.length(), "JUnit in Action")

    def testQueryParser(self):

        searcher = IndexSearcher(self.directory)

        query = QueryParser("contents",
                            SimpleAnalyzer()).parse("+JUNIT +ANT -MOCK")
        hits = searcher.search(query)
        self.assertEqual(1, hits.length())
        d = hits.doc(0)
        self.assertEqual("Java Development with Ant", d.get("title"))

        query = QueryParser("contents", SimpleAnalyzer()).parse("mock OR junit")
        hits = searcher.search(query)
        self.assertEqual(2, hits.length(), "JDwA and JIA")



# ====================================================================
# Copyright (c) 2004-2005 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions: 
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software. 
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#

import os

from PyLucene import \
     SimpleAnalyzer, Document, QueryParser, Explanation, \
     IndexSearcher, FSDirectory


class Explainer(object):

    def main(cls, argv):

        if len(argv) != 3:
            print "Usage: Explainer <index dir> <query>"

        else:
            indexDir = argv[1]
            queryExpression = argv[2]

            directory = FSDirectory.getDirectory(indexDir, False)

            query = QueryParser("contents",
                                SimpleAnalyzer()).parse(queryExpression)

            print "Query:", queryExpression

            searcher = IndexSearcher(directory)
            hits = searcher.search(query)

            for i, doc in hits:
                explanation = searcher.explain(query, hits.id(i))
                print "----------"
                print doc["title"].encode('utf-8')
                print explanation

    main = classmethod(main)


if __name__ == "__main__":
    import sys
    Explainer.main(sys.argv)



# ====================================================================
# Copyright (c) 2004-2005 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions: 
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software. 
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#

from unittest import TestCase, main
from PyLucene import *


class FuzzyQueryTestCase(TestCase):
    """
    Unit tests ported from Java Lucene
    """

    def _addDoc(self, text, writer):
        doc = Document()
        doc.add(Field("field", text,
                      Field.Store.YES, Field.Index.TOKENIZED))
        writer.addDocument(doc)

    def testDefaultFuzziness(self):

        directory = RAMDirectory()
        writer = IndexWriter(directory, WhitespaceAnalyzer(), True)
        self._addDoc("aaaaa", writer)
        self._addDoc("aaaab", writer)
        self._addDoc("aaabb", writer)
        self._addDoc("aabbb", writer)
        self._addDoc("abbbb", writer)
        self._addDoc("bbbbb", writer)
        self._addDoc("ddddd", writer)
        writer.optimize()
        writer.close()

        searcher = IndexSearcher(directory)

        query = FuzzyQuery(Term("field", "aaaaa"))
        hits = searcher.search(query)
        self.assertEqual(3, hits.length())

        # not similar enough:
        query = FuzzyQuery(Term("field", "xxxxx"))
        hits = searcher.search(query)
        self.assertEqual(0, hits.length())
        # edit distance to "aaaaa" = 3
        query = FuzzyQuery(Term("field", "aaccc"))
        hits = searcher.search(query)
        self.assertEqual(0, hits.length())

        # query identical to a word in the index:
        query = FuzzyQuery(Term("field", "aaaaa"))
        hits = searcher.search(query)
        self.assertEqual(3, hits.length())
        self.assertEqual(hits.doc(0).get("field"), "aaaaa")
        # default allows for up to two edits:
        self.assertEqual(hits.doc(1).get("field"), "aaaab")
        self.assertEqual(hits.doc(2).get("field"), "aaabb")

        # query similar to a word in the index:
        query = FuzzyQuery(Term("field", "aaaac"))
        hits = searcher.search(query)
        self.assertEqual(3, hits.length())
        self.assertEqual(hits.doc(0).get("field"), "aaaaa")
        self.assertEqual(hits.doc(1).get("field"), "aaaab")
        self.assertEqual(hits.doc(2).get("field"), "aaabb")

        query = FuzzyQuery(Term("field", "ddddX"))
        hits = searcher.search(query)
        self.assertEqual(1, hits.length())
        self.assertEqual(hits.doc(0).get("field"), "ddddd")

        # different field = no match:
        query = FuzzyQuery(Term("anotherfield", "ddddX"))
        hits = searcher.search(query)
        self.assertEqual(0, hits.length())

        searcher.close()
        directory.close()

    def testDefaultFuzzinessLong(self):

        directory = RAMDirectory()
        writer = IndexWriter(directory, WhitespaceAnalyzer(), True)
        self._addDoc("aaaaaaa", writer)
        self._addDoc("segment", writer)
        writer.optimize()
        writer.close()
        searcher = IndexSearcher(directory)

        # not similar enough:
        query = FuzzyQuery(Term("field", "xxxxx"))
        hits = searcher.search(query)
        self.assertEqual(0, hits.length())
        # edit distance to "aaaaaaa" = 3, this matches because
        # the string is longer than
        # in testDefaultFuzziness so a bigger difference is allowed:
        query = FuzzyQuery(Term("field", "aaaaccc"))
        hits = searcher.search(query)
        self.assertEqual(1, hits.length())
        self.assertEqual(hits.doc(0).get("field"), "aaaaaaa")

        # no match, more than half of the characters is wrong:
        query = FuzzyQuery(Term("field", "aaacccc"))
        hits = searcher.search(query)
        self.assertEqual(0, hits.length())

        # "student" and "stellent" are indeed similar to "segment" by default:
        query = FuzzyQuery(Term("field", "student"))
        hits = searcher.search(query)
        self.assertEqual(1, hits.length())
        query = FuzzyQuery(Term("field", "stellent"))
        hits = searcher.search(query)
        self.assertEqual(1, hits.length())

        searcher.close()
        directory.close()


if __name__ == "__main__":
    import sys
    if '-loop' in sys.argv:
        sys.argv.remove('-loop')
        while True:
            try:
                main()
            except:
                pass
    else:
         main()



 # ====================================================================
 # Copyright (c) 2004-2006 Open Source Applications Foundation.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
 # to deal in the Software without restriction, including without limitation
 # the rights to use, copy, modify, merge, publish, distribute, sublicense,
 # and/or sell copies of the Software, and to permit persons to whom the
 # Software is furnished to do so, subject to the following conditions: 
 #
 # The above copyright notice and this permission notice shall be included
 # in all copies or substantial portions of the Software. 
 #
 # THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 # OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 # FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 # AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 # LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 # FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 # DEALINGS IN THE SOFTWARE.
 # ====================================================================


class JavaError(Exception):
    def getJavaException(self):
        return self.args[0]


class InvalidArgsError(Exception):
    pass


from _PyLucene import *


import threading

# A few kludges here because starting the OS thread is done by libgcj
# instead of python.
    
class PythonThread(threading.Thread):
    """
    A threading.Thread extension that delegates starting of the
    actual OS thread to libgcj. In order to keep libgcj's garbage collector
    happy, any python thread using libgcj must be of this class.
    """

    def __init__(self, *args, **kwds):

        super(PythonThread, self).__init__(*args, **kwds)
        self.javaThread = None

    def start(self):

        current = threading.currentThread()
        assert (current.getName() == 'MainThread' or isinstance(current, PythonThread)), "PythonThread can only be started from main thread of from another PythonThread"
        
        class runnable(object):
            def __init__(_self, callable):
                _self.callable = callable
            def run(_self):
                try:
                    _self.callable()
                finally:
                    del _self.callable
                    self.javaThread = None

        threading._active_limbo_lock.acquire()
        threading._limbo[self] = self
        threading._active_limbo_lock.release()
        
        thread = self.javaThread = Thread(runnable(self._Thread__bootstrap),
                                          self.getName())
        thread.start()

        self._Thread__started = True

    def join(self, timeout=None):

        thread = self.javaThread
        if thread is not None:
            if timeout is not None:
                thread.join(long(timeout * 1000))
            else:
                thread.join()




import os, sys, tarfile

baseDir = os.path.dirname(os.path.abspath(sys.argv[0]))
sys.path.append(baseDir)

from lia.common.TestDataDocumentHandler import TestDataDocumentHandler

TestDataDocumentHandler.createIndex(os.path.join(baseDir, 'data'),
                                    os.path.join(baseDir, 'index'),
                                    False)

tar = tarfile.open(os.path.join(baseDir, 'indexes.tar.gz'), "r:gz")
while True:
    member = tar.next()
    if member is None:
        break
    print member.name
    tar.extract(member, baseDir)
tar.close()



# ====================================================================
# Copyright (c) 2004-2005 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions: 
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software. 
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#

from random import seed, randint
from unittest import TestCase, main

from PyLucene import \
     SimpleAnalyzer, Document, Field, IndexWriter, RAMDirectory, \
     IndexReader, IndexSearcher, Term, TermQuery, RangeFilter


class BaseTestRangeFilter(TestCase):

    def __init__(self, *args):

        super(BaseTestRangeFilter, self).__init__(*args)

        self.index = RAMDirectory()

        self.MAX_INT = 0x7fffffff
        self.maxR = ~self.MAX_INT
        self.minR = self.MAX_INT
    
        self.minId = 0
        self.maxId = 10000

        self.build()

    #
    # a simple padding function that should work with any int
    #

    def pad(self, n):

        if n < 0:
            return "-%0.10d" % (self.MAX_INT + n + 1)
        else:
            return "0%0.10d" % n

    def build(self):

        # build an index
        writer = IndexWriter(self.index, SimpleAnalyzer(), True)

        seed(101)
        for d in xrange(self.minId, self.maxId + 1):
            doc = Document()
            doc.add(Field("id", self.pad(d),
                          Field.Store.YES, Field.Index.UN_TOKENIZED))
            r = randint(~self.MAX_INT, self.MAX_INT)
            if self.maxR < r:
                self.maxR = r
            if r < self.minR:
                self.minR = r
            doc.add(Field("rand", self.pad(r),
                          Field.Store.YES, Field.Index.UN_TOKENIZED))
            doc.add(Field("body", "body",
                          Field.Store.YES, Field.Index.UN_TOKENIZED))
            writer.addDocument(doc)
            
        writer.optimize()
        writer.close()

    def testPad(self):

        tests = [-9999999, -99560, -100, -3, -1, 0, 3, 9, 10, 1000, 999999999]

        for i in xrange(0, len(tests) - 1):
            a = tests[i]
            b = tests[i + 1]
            aa = self.pad(a)
            bb = self.pad(b)
            label = "%s:%s vs %s:%s" %(a, aa, b, bb)
            self.assertEqual(len(aa), len(bb), "length of %s" %label)
            self.assert_(aa < bb, "compare less than %s" %label)


 #
 # A basic 'positive' Unit test class for the RangeFilter class.
 #
 # NOTE: at the moment, this class only tests for 'positive' results,
 # it does not verify the results to ensure there are no 'false positives',
 # nor does it adequately test 'negative' results.  It also does not test
 # that garbage in results in an Exception.
 #

class TestRangeFilter(BaseTestRangeFilter):

    def testRangeFilterId(self):

        reader = IndexReader.open(self.index);
        search = IndexSearcher(reader)

        medId = ((self.maxId - self.minId) / 2)
        
        minIP = self.pad(self.minId)
        maxIP = self.pad(self.maxId)
        medIP = self.pad(medId)
    
        numDocs = reader.numDocs()
        
        self.assertEqual(numDocs, 1 + self.maxId - self.minId, "num of docs")
        
        q = TermQuery(Term("body","body"))

        # test id, bounded on both ends
        
        result = search.search(q, RangeFilter("id", minIP, maxIP,
                                              True, True))
        self.assertEqual(numDocs, len(result), "find all")

        result = search.search(q, RangeFilter("id", minIP, maxIP,
                                              True, False))
        self.assertEqual(numDocs - 1, len(result), "all but last")

        result = search.search(q, RangeFilter("id", minIP, maxIP,
                                              False, True))
        self.assertEqual(numDocs - 1, len(result), "all but first")
        
        result = search.search(q, RangeFilter("id", minIP, maxIP,
                                              False, False))
        self.assertEqual(numDocs - 2, len(result), "all but ends")
    
        result = search.search(q, RangeFilter("id", medIP, maxIP,
                                              True, True))
        self.assertEqual(1 + self.maxId - medId, len(result), "med and up")
        
        result = search.search(q, RangeFilter("id", minIP, medIP,
                                              True, True))
        self.assertEqual(1 + medId - self.minId, len(result), "up to med")

        # unbounded id

        result = search.search(q, RangeFilter("id", minIP, None,
                                              True, False))
        self.assertEqual(numDocs, len(result), "min and up")

        result = search.search(q, RangeFilter("id", None, maxIP,
                                              False, True))
        self.assertEqual(numDocs, len(result), "max and down")

        result = search.search(q, RangeFilter("id", minIP, None,
                                              False, False))
        self.assertEqual(numDocs - 1, len(result), "not min, but up")
        
        result = search.search(q, RangeFilter("id", None, maxIP,
                                              False, False))
        self.assertEqual(numDocs - 1, len(result), "not max, but down")
        
        result = search.search(q, RangeFilter("id",medIP, maxIP,
                                              True, False))
        self.assertEqual(self.maxId - medId, len(result), "med and up, not max")
        
        result = search.search(q, RangeFilter("id", minIP, medIP,
                                              False, True))
        self.assertEqual(medId - self.minId, len(result), "not min, up to med")

        # very small sets

        result = search.search(q, RangeFilter("id", minIP, minIP,
                                              False, False))
        self.assertEqual(0, len(result), "min, min, False, False")

        result = search.search(q, RangeFilter("id", medIP, medIP,
                                              False, False))
        self.assertEqual(0, len(result), "med, med, False, False")
        result = search.search(q, RangeFilter("id", maxIP, maxIP,
                                              False, False))
        self.assertEqual(0, len(result), "max, max, False, False")
                     
        result = search.search(q, RangeFilter("id", minIP, minIP,
                                              True, True))
        self.assertEqual(1, len(result), "min, min, True, True")
        result = search.search(q, RangeFilter("id", None, minIP,
                                              False, True))
        self.assertEqual(1, len(result), "nul, min, False, True")

        result = search.search(q, RangeFilter("id", maxIP, maxIP,
                                              True, True))
        self.assertEqual(1, len(result), "max, max, True, True")
        result = search.search(q, RangeFilter("id", maxIP, None,
                                              True, False))
        self.assertEqual(1, len(result), "max, nul, True, True")

        result = search.search(q, RangeFilter("id", medIP, medIP,
                                              True, True))
        self.assertEqual(1, len(result), "med, med, True, True")

    def testRangeFilterRand(self):

        reader = IndexReader.open(self.index)
        search = IndexSearcher(reader)

        minRP = self.pad(self.minR)
        maxRP = self.pad(self.maxR)

        numDocs = reader.numDocs()
        
        self.assertEqual(numDocs, 1 + self.maxId - self.minId, "num of docs")
        
        q = TermQuery(Term("body", "body"))

        # test extremes, bounded on both ends
        
        result = search.search(q, RangeFilter("rand", minRP, maxRP,
                                              True, True))
        self.assertEqual(numDocs, len(result), "find all")

        result = search.search(q, RangeFilter("rand", minRP, maxRP,
                                              True, False))
        self.assertEqual(numDocs - 1, len(result), "all but biggest")

        result = search.search(q, RangeFilter("rand", minRP, maxRP,
                                              False, True))
        self.assertEqual(numDocs - 1, len(result), "all but smallest")
        
        result = search.search(q, RangeFilter("rand", minRP, maxRP,
                                              False, False))
        self.assertEqual(numDocs - 2, len(result), "all but extremes")
    
        # unbounded

        result = search.search(q, RangeFilter("rand", minRP, None,
                                              True, False))
        self.assertEqual(numDocs, len(result), "smallest and up")

        result = search.search(q, RangeFilter("rand", None, maxRP,
                                              False, True))
        self.assertEqual(numDocs, len(result), "biggest and down")

        result = search.search(q, RangeFilter("rand", minRP, None,
                                              False, False))
        self.assertEqual(numDocs - 1, len(result), "not smallest, but up")
        
        result = search.search(q, RangeFilter("rand", None, maxRP,
                                              False, False))
        self.assertEqual(numDocs - 1, len(result), "not biggest, but down")
        
        # very small sets

        result = search.search(q, RangeFilter("rand", minRP, minRP,
                                              False, False))
        self.assertEqual(0, len(result), "min, min, False, False")

        result = search.search(q, RangeFilter("rand", maxRP, maxRP,
                                              False, False))
        self.assertEqual(0, len(result), "max, max, False, False")
                     
        result = search.search(q, RangeFilter("rand", minRP, minRP,
                                              True, True))
        self.assertEqual(1, len(result), "min, min, True, True")

        result = search.search(q, RangeFilter("rand", None, minRP,
                                              False, True))
        self.assertEqual(1, len(result), "nul, min, False, True")

        result = search.search(q, RangeFilter("rand", maxRP, maxRP,
                                              True, True))
        self.assertEqual(1, len(result), "max, max, True, True")

        result = search.search(q, RangeFilter("rand", maxRP, None,
                                              True, False))
        self.assertEqual(1, len(result), "max, nul, True, True")


if __name__ == "__main__":
    import sys
    if '-loop' in sys.argv:
        sys.argv.remove('-loop')
        while True:
            try:
                main(defaultTest='TestRangeFilter')
            except:
                pass
    else:
        main(defaultTest='TestRangeFilter')



# ====================================================================
# Copyright (c) 2004-2005 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions: 
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software. 
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#

import unittest
from PyLucene import *

class Test_Bug1763(unittest.TestCase):

    def setUp(self):

        self.analyzer = StandardAnalyzer()
        self.d1 = RAMDirectory()
        self.d2 = RAMDirectory()
        
        w1, w2 = [IndexWriter(d, self.analyzer, True)
                  for d in [self.d1, self.d2]]
        doc1 = Document()
        doc2 = Document()
        doc1.add(Field("all", "blah blah double blah Gesundheit",
                       Field.Store.NO, Field.Index.TOKENIZED))
        doc1.add(Field('id', '1', Field.Store.YES, Field.Index.NO))
        doc2.add(Field("all", "a quick brown test ran over the lazy data",
                       Field.Store.NO, Field.Index.TOKENIZED))
        doc2.add(Field('id', '2',
                       Field.Store.YES, Field.Index.NO))
        w1.addDocument(doc1)
        w2.addDocument(doc2)
        for w in [w1, w2]:
            w.optimize()
            w.close()

    def tearDown(self):
        pass

    def test_bug1763(self):
            
        w1 = IndexWriter(self.d1, self.analyzer, True)
        w1.addIndexes([self.d2])
        w1.optimize()
        w1.close()

        searcher = IndexSearcher(self.d1)
        q = QueryParser('all', self.analyzer).parse('brown')
        hits = searcher.search(q)
        self.assertEqual(hits.doc(0).get('id'), '2')


if __name__ == '__main__':
    unittest.main()



# ====================================================================
# Copyright (c) 2004-2005 Open Source Applications Foundation.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions: 
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software. 
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ====================================================================
#

from PyLucene import \
     RAMDirectory, IndexWriter, StandardAnalyzer, Document, Field, \
     QueryParser

class UsingAnalyzersExample(object):

    #
    # This method doesn't do anything, except compile correctly.
    # This is used to show snippets of how Analyzers are used.
    #
    def someMethod(self):

        directory = RAMDirectory()

        analyzer = StandardAnalyzer()
        writer = IndexWriter(directory, analyzer, True)

        doc = Document()
        doc.add(Field.Text("title", "This is the title"))
        doc.add(Field.UnStored("contents", "...document contents..."))
        writer.addDocument(doc)

        writer.addDocument(doc, analyzer)

        expression = "some query"

        query = QueryParser.parse(expression, "contents", analyzer)

        parser = QueryParser("contents", analyzer)
        query = parser.parseQuery(expression)
